# encoding: utf-8
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import time

import pytest
import salt.config
import salt.netapi
from salt.exceptions import EauthAuthenticationError
from tests.support.case import SSHCase
from tests.support.helpers import (
    SaveRequestsPostHandler,
    Webserver,
    requires_sshd_server,
    slowTest,
)
from tests.support.mock import patch
from tests.support.paths import TMP, TMP_CONF_DIR
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf

log = logging.getLogger(__name__)


@pytest.mark.usefixtures("salt_sub_minion")
class NetapiClientTest(TestCase):
    eauth_creds = {
        "username": "saltdev_auto",
        "password": "saltdev",
        "eauth": "auto",
    }

    def setUp(self):
        """
        Set up a NetapiClient instance
        """
        opts = salt.config.client_config(
            os.path.join(RUNTIME_VARS.TMP_CONF_DIR, "master")
        )
        self.netapi = salt.netapi.NetapiClient(opts)

    def tearDown(self):
        del self.netapi

    @slowTest
    def test_local(self):
        low = {"client": "local", "tgt": "*", "fun": "test.ping", "timeout": 300}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)
        # If --proxy is set, it will cause an extra minion_id to be in the
        # response. Since there's not a great way to know if the test
        # runner's proxy minion is running, and we're not testing proxy
        # minions here anyway, just remove it from the response.
        ret.pop("proxytest", None)
        self.assertEqual(ret, {"minion": True, "sub_minion": True})

    @slowTest
    def test_local_batch(self):
        low = {"client": "local_batch", "tgt": "*", "fun": "test.ping", "timeout": 300}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)
        rets = []
        for _ret in ret:
            rets.append(_ret)
        self.assertIn({"sub_minion": True}, rets)
        self.assertIn({"minion": True}, rets)

    def test_local_async(self):
        low = {"client": "local_async", "tgt": "*", "fun": "test.ping"}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

        # Remove all the volatile values before doing the compare.
        self.assertIn("jid", ret)
        ret.pop("jid", None)
        ret["minions"] = sorted(ret["minions"])
        try:
            # If --proxy is set, it will cause an extra minion_id to be in the
            # response. Since there's not a great way to know if the test
            # runner's proxy minion is running, and we're not testing proxy
            # minions here anyway, just remove it from the response.
            ret["minions"].remove("proxytest")
        except ValueError:
            pass
        self.assertEqual(ret, {"minions": sorted(["minion", "sub_minion"])})

    def test_local_unauthenticated(self):
        low = {"client": "local", "tgt": "*", "fun": "test.ping"}

        with self.assertRaises(EauthAuthenticationError) as excinfo:
            ret = self.netapi.run(low)

    @slowTest
    def test_wheel(self):
        low = {"client": "wheel", "fun": "key.list_all"}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

        # Remove all the volatile values before doing the compare.
        self.assertIn("tag", ret)
        ret.pop("tag")

        data = ret.get("data", {})
        self.assertIn("jid", data)
        data.pop("jid", None)

        self.assertIn("tag", data)
        data.pop("tag", None)

        ret.pop("_stamp", None)
        data.pop("_stamp", None)

        self.maxDiff = None
        self.assertTrue(
            set(["master.pem", "master.pub"]).issubset(
                set(ret["data"]["return"]["local"])
            )
        )

    @slowTest
    def test_wheel_async(self):
        # Give this test a little breathing room
        time.sleep(3)
        low = {"client": "wheel_async", "fun": "key.list_all"}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)
        self.assertIn("jid", ret)
        self.assertIn("tag", ret)

    def test_wheel_unauthenticated(self):
        low = {"client": "wheel", "tgt": "*", "fun": "test.ping"}

        with self.assertRaises(EauthAuthenticationError) as excinfo:
            ret = self.netapi.run(low)

    @skipIf(True, "This is not testing anything. Skipping for now.")
    def test_runner(self):
        # TODO: fix race condition in init of event-- right now the event class
        # will finish init even if the underlying zmq socket hasn't connected yet
        # this is problematic for the runnerclient's master_call method if the
        # runner is quick
        # low = {'client': 'runner', 'fun': 'cache.grains'}
        low = {"client": "runner", "fun": "test.sleep", "arg": [2]}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

    @skipIf(True, "This is not testing anything. Skipping for now.")
    def test_runner_async(self):
        low = {"client": "runner", "fun": "cache.grains"}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

    def test_runner_unauthenticated(self):
        low = {"client": "runner", "tgt": "*", "fun": "test.ping"}

        with self.assertRaises(EauthAuthenticationError) as excinfo:
            ret = self.netapi.run(low)


@requires_sshd_server
class NetapiSSHClientTest(SSHCase):
    eauth_creds = {
        "username": "saltdev_auto",
        "password": "saltdev",
        "eauth": "auto",
    }

    def setUp(self):
        """
        Set up a NetapiClient instance
        """
        opts = salt.config.client_config(os.path.join(TMP_CONF_DIR, "master"))
        self.netapi = salt.netapi.NetapiClient(opts)

        self.priv_file = os.path.join(RUNTIME_VARS.TMP_CONF_DIR, "key_test")
        self.rosters = os.path.join(RUNTIME_VARS.TMP_CONF_DIR)

        # Initialize salt-ssh
        self.run_function("test.ping")

    def tearDown(self):
        del self.netapi

    @classmethod
    def setUpClass(cls):
        cls.post_webserver = Webserver(handler=SaveRequestsPostHandler)
        cls.post_webserver.start()
        cls.post_web_root = cls.post_webserver.web_root
        cls.post_web_handler = cls.post_webserver.handler

    @classmethod
    def tearDownClass(cls):
        cls.post_webserver.stop()
        del cls.post_webserver

    @slowTest
    def test_ssh(self):
        low = {
            "client": "ssh",
            "tgt": "localhost",
            "fun": "test.ping",
            "ignore_host_keys": True,
            "roster_file": "roster",
            "rosters": [self.rosters],
            "ssh_priv": self.priv_file,
        }

        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

        self.assertIn("localhost", ret)
        self.assertIn("return", ret["localhost"])
        self.assertEqual(ret["localhost"]["return"], True)
        self.assertEqual(ret["localhost"]["id"], "localhost")
        self.assertEqual(ret["localhost"]["fun"], "test.ping")

    @slowTest
    def test_ssh_unauthenticated(self):
        low = {"client": "ssh", "tgt": "localhost", "fun": "test.ping"}

        with self.assertRaises(EauthAuthenticationError) as excinfo:
            ret = self.netapi.run(low)

    @slowTest
    def test_ssh_unauthenticated_raw_shell_curl(self):

        fun = "-o ProxyCommand curl {0}".format(self.post_web_root)
        low = {"client": "ssh", "tgt": "localhost", "fun": fun, "raw_shell": True}

        ret = None
        with self.assertRaises(EauthAuthenticationError) as excinfo:
            ret = self.netapi.run(low)

        self.assertEqual(self.post_web_handler.received_requests, [])
        self.assertEqual(ret, None)

    @slowTest
    def test_ssh_unauthenticated_raw_shell_touch(self):

        badfile = os.path.join(TMP, "badfile.txt")
        fun = "-o ProxyCommand touch {0}".format(badfile)
        low = {"client": "ssh", "tgt": "localhost", "fun": fun, "raw_shell": True}

        ret = None
        with self.assertRaises(EauthAuthenticationError) as excinfo:
            ret = self.netapi.run(low)

        self.assertEqual(ret, None)
        self.assertFalse(os.path.exists("badfile.txt"))

    @slowTest
    def test_ssh_authenticated_raw_shell_disabled(self):

        badfile = os.path.join(TMP, "badfile.txt")
        fun = "-o ProxyCommand touch {0}".format(badfile)
        low = {"client": "ssh", "tgt": "localhost", "fun": fun, "raw_shell": True}

        low.update(self.eauth_creds)

        ret = None
        with patch.dict(self.netapi.opts, {"netapi_allow_raw_shell": False}):
            with self.assertRaises(EauthAuthenticationError) as excinfo:
                ret = self.netapi.run(low)

        self.assertEqual(ret, None)
        self.assertFalse(os.path.exists("badfile.txt"))



# -*- coding: utf-8 -*-
"""
Renderer to test argline handling in jinja renderer

See: https://github.com/saltstack/salt/issues/55124
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import salt libs
import salt.utils.stringio


def render(gpg_data, saltenv="base", sls="", argline="", **kwargs):
    """
    Renderer which returns the text value of the SLS file, instead of a
    StringIO object.
    """
    if salt.utils.stringio.is_readable(gpg_data):
        return gpg_data.getvalue()
    else:
        return gpg_data



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.at as at

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AtTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.at
    """

    def setup_loader_modules(self):
        return {at: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to add a job to queue.
        """

        # input variables
        name = "jboss"
        timespec = "9:09 11/04/15"
        tag = "love"
        user = "jam"

        # mock for at.at module call
        mock_atat = {
            "jobs": [
                {
                    "date": "2015-11-04",
                    "job": "1476031633.a",
                    "queue": "a",
                    "tag": tag,
                    "time": "09:09:00",
                    "user": user,
                },
            ],
        }

        # normale return
        ret = {
            "name": name,
            "result": True,
            "changes": {
                "date": "2015-11-04",
                "job": "1476031633.a",
                "queue": "a",
                "tag": "love",
                "time": "09:09:00",
                "user": "jam",
            },
            "comment": "job {name} added and will run on {timespec}".format(
                name=name, timespec=timespec
            ),
        }

        # unknown user return
        ret_user = {}
        ret_user.update(ret)
        ret_user.update(
            {
                "result": False,
                "changes": {},
                "comment": "user {0} does not exists".format(user),
            }
        )

        # add a job
        mock = MagicMock(return_value=mock_atat)
        with patch.dict(at.__opts__, {"test": False}):
            with patch.dict(at.__salt__, {"at.at": mock}):
                self.assertDictEqual(at.present(name, timespec, tag), ret)

        # add a job with a non-existing user
        mock = MagicMock(return_value=False)
        with patch.dict(at.__opts__, {"test": False}):
            with patch.dict(at.__salt__, {"user.info": mock}):
                self.assertDictEqual(at.present(name, timespec, tag, user), ret_user)

        # add a job with test=True
        with patch.dict(at.__opts__, {"test": True}):
            ret_test = {}
            ret_test.update(ret)
            ret_test.update({"result": None, "changes": {}})
            self.assertDictEqual(at.present(name, timespec, tag, user), ret_test)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to remove a job from queue
        """
        # input variables
        name = "jboss"
        tag = "rose"
        user = "jam"

        # mock for at.atrm module call
        mock_atatrm = {
            "jobs": {"removed": ["1476033859.a", "1476033855.a"], "tag": None},
        }

        # mock for at.jobcheck module call
        mock_atjobcheck = {
            "jobs": [
                {
                    "date": "2015-11-04",
                    "job": "1476031633.a",
                    "queue": "a",
                    "tag": tag,
                    "time": "09:09:00",
                    "user": user,
                },
            ],
        }

        # normal return
        ret = {
            "name": name,
            "result": True,
            "changes": {"removed": ["1476033859.a", "1476033855.a"]},
            "comment": "removed 2 job(s)",
        }

        # remove a job with test=True
        with patch.dict(at.__opts__, {"test": True}):
            ret_test = {}
            ret_test.update(ret)
            ret_test.update(
                {"result": None, "changes": {}, "comment": "removed ? job(s)"}
            )
            self.assertDictEqual(at.absent(name), ret_test)

        # remove a job and pass limit parameter
        with patch.dict(at.__opts__, {"test": False}):
            ret_limit = {}
            ret_limit.update(ret)
            ret_limit.update(
                {
                    "result": False,
                    "changes": {},
                    "comment": "limit parameter not supported {0}".format(name),
                }
            )
            self.assertDictEqual(at.absent(name, limit="all"), ret_limit)

        # remove all jobs (2 jobs found)
        mock = MagicMock(return_value=mock_atatrm)
        with patch.dict(at.__salt__, {"at.atrm": mock}):
            with patch.dict(at.__opts__, {"test": False}):
                self.assertDictEqual(at.absent(name), ret)

        # remove all jobs (0 jobs found)
        mock_atatrm_nojobs = {}
        mock_atatrm_nojobs.update(mock_atatrm)
        mock_atatrm_nojobs.update({"jobs": {"removed": []}})
        mock = MagicMock(return_value=mock_atatrm_nojobs)
        with patch.dict(at.__salt__, {"at.atrm": mock}):
            with patch.dict(at.__opts__, {"test": False}):
                ret_nojobs = {}
                ret_nojobs.update(ret)
                ret_nojobs.update(
                    {"changes": {}, "comment": ret["comment"].replace("2", "0")}
                )
                self.assertDictEqual(at.absent(name), ret_nojobs)

        # remove all tagged jobs (1 jobs found)
        mock_atatrm_tag = {}
        mock_atatrm_tag.update(mock_atatrm)
        mock_atatrm_tag.update({"jobs": {"removed": ["1476031633.a"], "tag": "rose"}})
        mock = MagicMock(return_value=mock_atatrm_tag)
        with patch.dict(at.__salt__, {"at.atrm": mock}):
            mock = MagicMock(return_value=mock_atjobcheck)
            with patch.dict(at.__salt__, {"at.jobcheck": mock}):
                with patch.dict(at.__opts__, {"test": False}):
                    ret_tag = {}
                    ret_tag.update(ret)
                    ret_tag.update(
                        {
                            "changes": {"removed": ["1476031633.a"]},
                            "comment": ret["comment"].replace("2", "1"),
                        }
                    )
                    self.assertDictEqual(at.absent(name, tag=tag), ret_tag)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.aptpkg as aptpkg

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AptTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.aptpkg
    """

    def setup_loader_modules(self):
        return {aptpkg: {}}

    # 'held' function tests: 1

    def test_held(self):
        """
        Test to set package in 'hold' state, meaning it will not be upgraded.
        """
        name = "tmux"

        ret = {
            "name": name,
            "result": False,
            "changes": {},
            "comment": "Package {0} does not have a state".format(name),
        }

        mock = MagicMock(return_value=False)
        with patch.dict(aptpkg.__salt__, {"pkg.get_selections": mock}):
            self.assertDictEqual(aptpkg.held(name), ret)



# coding: utf-8

# Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Salt libs
import salt.beacons.smartos_imgadm as imgadm
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase

# Mock Results
MOCK_CLEAN_STATE = {"first_run": True, "images": []}
MOCK_IMAGE_NONE = {}
MOCK_IMAGE_ONE = {
    "00000000-0000-0000-0000-000000000001": {
        "description": "Example Image 1",
        "name": "example-1",
        "os": "smartos",
        "published": "2018-01-01T00:42:00Z",
        "source": "https://images.joyent.com",
        "version": "18.1.0",
    },
}
MOCK_IMAGE_TWO = {
    "00000000-0000-0000-0000-000000000001": {
        "description": "Example Image 1",
        "name": "example-1",
        "os": "smartos",
        "published": "2018-01-01T00:42:00Z",
        "source": "https://images.joyent.com",
        "version": "18.1.0",
    },
    "00000000-0000-0000-0000-000000000002": {
        "description": "Example Image 2",
        "name": "example-2",
        "os": "smartos",
        "published": "2018-01-01T00:42:00Z",
        "source": "https://images.joyent.com",
        "version": "18.2.0",
    },
}


class SmartOSImgAdmBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.imgadm
    """

    def setup_loader_modules(self):
        return {imgadm: {"__context__": {}, "__salt__": {}}}

    def test_non_list_config(self):
        """
        We only have minimal validation so we test that here
        """
        assert imgadm.validate({}) == (
            False,
            "Configuration for imgadm beacon must be a list!",
        )

    def test_imported_startup(self):
        """
        Test with one image and startup_import_event
        """
        # NOTE: this should yield 1 imported event
        with patch.dict(imgadm.IMGADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            imgadm.__salt__, {"imgadm.list": MagicMock(return_value=MOCK_IMAGE_ONE)}
        ):

            config = [{"startup_import_event": True}]
            assert imgadm.validate(config) == (True, "Valid beacon configuration")

            ret = imgadm.beacon(config)
            res = [
                {
                    "description": "Example Image 1",
                    "name": "example-1",
                    "os": "smartos",
                    "published": "2018-01-01T00:42:00Z",
                    "source": "https://images.joyent.com",
                    "tag": "imported/00000000-0000-0000-0000-000000000001",
                    "version": "18.1.0",
                }
            ]
            assert ret == res

    def test_imported_nostartup(self):
        """
        Test with one image and startup_import_event unset/false
        """
        # NOTE: this should yield 0 imported event
        with patch.dict(imgadm.IMGADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            imgadm.__salt__, {"imgadm.list": MagicMock(return_value=MOCK_IMAGE_ONE)}
        ):

            config = []

            assert imgadm.validate(config) == (True, "Valid beacon configuration")
            assert imgadm.beacon(config) == []

    def test_imported(self):
        """
        Test with one image and a new image added on the 2nd pass
        """
        # NOTE: this should yield 1 imported event
        with patch.dict(imgadm.IMGADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            imgadm.__salt__,
            {"imgadm.list": MagicMock(side_effect=[MOCK_IMAGE_ONE, MOCK_IMAGE_TWO])},
        ):

            config = []
            assert imgadm.validate(config) == (True, "Valid beacon configuration")

            # Initial pass (Initialized state and do not yield imported images at startup)
            imgadm.beacon(config)

            # Second pass (After importing a new image)
            ret = imgadm.beacon(config)
            res = [
                {
                    "description": "Example Image 2",
                    "name": "example-2",
                    "os": "smartos",
                    "published": "2018-01-01T00:42:00Z",
                    "source": "https://images.joyent.com",
                    "tag": "imported/00000000-0000-0000-0000-000000000002",
                    "version": "18.2.0",
                }
            ]

            assert ret == res

    def test_deleted(self):
        """
        Test with two images and one gets deletes
        """
        # NOTE: this should yield 1 deleted event
        with patch.dict(imgadm.IMGADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            imgadm.__salt__,
            {"imgadm.list": MagicMock(side_effect=[MOCK_IMAGE_TWO, MOCK_IMAGE_ONE])},
        ):

            config = []
            assert imgadm.validate(config) == (True, "Valid beacon configuration")

            # Initial pass (Initialized state and do not yield imported images at startup)
            imgadm.beacon(config)

            # Second pass (After deleting one image)
            ret = imgadm.beacon(config)
            res = [
                {
                    "description": "Example Image 2",
                    "name": "example-2",
                    "os": "smartos",
                    "published": "2018-01-01T00:42:00Z",
                    "source": "https://images.joyent.com",
                    "tag": "deleted/00000000-0000-0000-0000-000000000002",
                    "version": "18.2.0",
                }
            ]

            assert ret == res

    def test_complex(self):
        """
        Test with one image, delete both, import 2
        """
        # NOTE: this should yield 1 delete and 2 import events
        with patch.dict(imgadm.IMGADM_STATE, MOCK_CLEAN_STATE), patch.dict(
            imgadm.__salt__,
            {
                "imgadm.list": MagicMock(
                    side_effect=[MOCK_IMAGE_ONE, MOCK_IMAGE_NONE, MOCK_IMAGE_TWO]
                )
            },
        ):

            config = []
            assert imgadm.validate(config), (True, "Valid beacon configuration")

            # Initial pass (Initialized state and do not yield imported images at startup)
            imgadm.beacon(config)

            # Second pass (After deleting one image)
            ret = imgadm.beacon(config)
            res = [
                {
                    "description": "Example Image 1",
                    "name": "example-1",
                    "os": "smartos",
                    "published": "2018-01-01T00:42:00Z",
                    "source": "https://images.joyent.com",
                    "tag": "deleted/00000000-0000-0000-0000-000000000001",
                    "version": "18.1.0",
                }
            ]
            assert ret == res

            # Third pass (After importing two images)
            ret = imgadm.beacon(config)
            res = [
                {
                    "description": "Example Image 1",
                    "name": "example-1",
                    "os": "smartos",
                    "published": "2018-01-01T00:42:00Z",
                    "source": "https://images.joyent.com",
                    "tag": "imported/00000000-0000-0000-0000-000000000001",
                    "version": "18.1.0",
                },
                {
                    "description": "Example Image 2",
                    "name": "example-2",
                    "os": "smartos",
                    "published": "2018-01-01T00:42:00Z",
                    "source": "https://images.joyent.com",
                    "tag": "imported/00000000-0000-0000-0000-000000000002",
                    "version": "18.2.0",
                },
            ]
            assert len(ret) == 2
            for item in ret:
                assert item in res



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.unit.payload_test
    ~~~~~~~~~~~~~~~~~~~~~~~
"""
from __future__ import absolute_import, print_function, unicode_literals

import copy
import datetime
import errno
import logging
import threading
import time

import salt.exceptions
import salt.payload
import zmq
from salt.ext import six
from salt.utils import immutabletypes
from salt.utils.odict import OrderedDict
from tests.support.helpers import slowTest
from tests.support.unit import TestCase, skipIf

log = logging.getLogger(__name__)


class PayloadTestCase(TestCase):
    def assertNoOrderedDict(self, data):
        if isinstance(data, OrderedDict):
            raise AssertionError("Found an ordered dictionary")
        if isinstance(data, dict):
            for value in six.itervalues(data):
                self.assertNoOrderedDict(value)
        elif isinstance(data, (list, tuple)):
            for chunk in data:
                self.assertNoOrderedDict(chunk)

    def test_list_nested_odicts(self):
        payload = salt.payload.Serial("msgpack")
        idata = {"pillar": [OrderedDict(environment="dev")]}
        odata = payload.loads(payload.dumps(idata.copy()))
        self.assertNoOrderedDict(odata)
        self.assertEqual(idata, odata)

    def test_datetime_dump_load(self):
        """
        Check the custom datetime handler can understand itself
        """
        payload = salt.payload.Serial("msgpack")
        dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
        idata = {dtvalue: dtvalue}
        sdata = payload.dumps(idata.copy())
        odata = payload.loads(sdata)
        self.assertEqual(
            sdata,
            b"\x81\xc7\x18N20010203T04:05:06.000007\xc7\x18N20010203T04:05:06.000007",
        )
        self.assertEqual(idata, odata)

    def test_verylong_dump_load(self):
        """
        Test verylong encoder/decoder
        """
        payload = salt.payload.Serial("msgpack")
        idata = {"jid": 20180227140750302662}
        sdata = payload.dumps(idata.copy())
        odata = payload.loads(sdata)
        idata["jid"] = "{0}".format(idata["jid"])
        self.assertEqual(idata, odata)

    def test_immutable_dict_dump_load(self):
        """
        Test immutable dict encoder/decoder
        """
        payload = salt.payload.Serial("msgpack")
        idata = {"dict": {"key": "value"}}
        sdata = payload.dumps({"dict": immutabletypes.ImmutableDict(idata["dict"])})
        odata = payload.loads(sdata)
        self.assertEqual(idata, odata)

    def test_immutable_list_dump_load(self):
        """
        Test immutable list encoder/decoder
        """
        payload = salt.payload.Serial("msgpack")
        idata = {"list": [1, 2, 3]}
        sdata = payload.dumps({"list": immutabletypes.ImmutableList(idata["list"])})
        odata = payload.loads(sdata)
        self.assertEqual(idata, odata)

    def test_immutable_set_dump_load(self):
        """
        Test immutable set encoder/decoder
        """
        payload = salt.payload.Serial("msgpack")
        idata = {"set": ["red", "green", "blue"]}
        sdata = payload.dumps({"set": immutabletypes.ImmutableSet(idata["set"])})
        odata = payload.loads(sdata)
        self.assertEqual(idata, odata)

    def test_odict_dump_load(self):
        """
        Test odict just works. It wasn't until msgpack 0.2.0
        """
        payload = salt.payload.Serial("msgpack")
        data = OrderedDict()
        data["a"] = "b"
        data["y"] = "z"
        data["j"] = "k"
        data["w"] = "x"
        sdata = payload.dumps({"set": data})
        odata = payload.loads(sdata)
        self.assertEqual({"set": dict(data)}, odata)

    def test_mixed_dump_load(self):
        """
        Test we can handle all exceptions at once
        """
        payload = salt.payload.Serial("msgpack")
        dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
        od = OrderedDict()
        od["a"] = "b"
        od["y"] = "z"
        od["j"] = "k"
        od["w"] = "x"
        idata = {
            dtvalue: dtvalue,  # datetime
            "jid": 20180227140750302662,  # long int
            "dict": immutabletypes.ImmutableDict({"key": "value"}),  # immutable dict
            "list": immutabletypes.ImmutableList([1, 2, 3]),  # immutable list
            "set": immutabletypes.ImmutableSet(
                ("red", "green", "blue")
            ),  # immutable set
            "odict": od,  # odict
        }
        edata = {
            dtvalue: dtvalue,  # datetime, == input
            "jid": "20180227140750302662",  # string repr of long int
            "dict": {"key": "value"},  # builtin dict
            "list": [1, 2, 3],  # builtin list
            "set": ["red", "green", "blue"],  # builtin set
            "odict": dict(od),  # builtin dict
        }
        sdata = payload.dumps(idata)
        odata = payload.loads(sdata)
        self.assertEqual(edata, odata)

    def test_recursive_dump_load(self):
        """
        Test recursive payloads are (mostly) serialized
        """
        payload = salt.payload.Serial("msgpack")
        data = {"name": "roscivs"}
        data["data"] = data  # Data all the things!
        sdata = payload.dumps(data)
        odata = payload.loads(sdata)
        self.assertTrue("recursion" in odata["data"].lower())

    def test_recursive_dump_load_with_identical_non_recursive_types(self):
        """
        If identical objects are nested anywhere, they should not be
        marked recursive unless they're one of the types we iterate
        over.
        """
        payload = salt.payload.Serial("msgpack")
        repeating = "repeating element"
        data = {
            "a": "a",  # Test CPython implementation detail. Short
            "b": "a",  # strings are interned.
            "c": 13,  # So are small numbers.
            "d": 13,
            "fnord": repeating,
            # Let's go for broke and make a crazy nested structure
            "repeating": [
                [[[[{"one": repeating, "two": repeating}], repeating, 13, "a"]]],
                repeating,
                repeating,
                repeating,
            ],
        }
        # We need a nested dictionary to trigger the exception
        data["repeating"][0][0][0].append(data)
        # If we don't deepcopy the data it gets mutated
        sdata = payload.dumps(copy.deepcopy(data))
        odata = payload.loads(sdata)
        # Delete the recursive piece - it's served its purpose, and our
        # other test tests that it's actually marked as recursive.
        del odata["repeating"][0][0][0][-1], data["repeating"][0][0][0][-1]
        self.assertDictEqual(odata, data)


class SREQTestCase(TestCase):
    port = 8845  # TODO: dynamically assign a port?

    @classmethod
    def setUpClass(cls):
        """
        Class to set up zmq echo socket
        """

        def echo_server():
            """
            A server that echos the message sent to it over zmq

            Optional "sleep" can be sent to delay response
            """
            context = zmq.Context()
            socket = context.socket(zmq.REP)
            socket.bind("tcp://*:{0}".format(SREQTestCase.port))
            payload = salt.payload.Serial("msgpack")

            while SREQTestCase.thread_running.is_set():
                try:
                    #  Wait for next request from client
                    message = socket.recv(zmq.NOBLOCK)
                    msg_deserialized = payload.loads(message)
                    log.info("Echo server received message: %s", msg_deserialized)
                    if isinstance(msg_deserialized["load"], dict) and msg_deserialized[
                        "load"
                    ].get("sleep"):
                        log.info(
                            "Test echo server sleeping for %s seconds",
                            msg_deserialized["load"]["sleep"],
                        )
                        time.sleep(msg_deserialized["load"]["sleep"])
                    socket.send(message)
                except zmq.ZMQError as exc:
                    if exc.errno == errno.EAGAIN:
                        continue
                    raise

        SREQTestCase.thread_running = threading.Event()
        SREQTestCase.thread_running.set()
        SREQTestCase.echo_server = threading.Thread(target=echo_server)
        SREQTestCase.echo_server.start()

    @classmethod
    def tearDownClass(cls):
        """
        Remove echo server
        """
        # kill the thread
        SREQTestCase.thread_running.clear()
        SREQTestCase.echo_server.join()

    def get_sreq(self):
        return salt.payload.SREQ("tcp://127.0.0.1:{0}".format(SREQTestCase.port))

    @slowTest
    def test_send_auto(self):
        """
        Test creation, send/rect
        """
        sreq = self.get_sreq()
        # check default of empty load and enc clear
        assert sreq.send_auto({}) == {"enc": "clear", "load": {}}

        # check that the load always gets passed
        assert sreq.send_auto({"load": "foo"}) == {"load": "foo", "enc": "clear"}

    def test_send(self):
        sreq = self.get_sreq()
        assert sreq.send("clear", "foo") == {"enc": "clear", "load": "foo"}

    @skipIf(True, "Disabled until we can figure out how to make this more reliable.")
    def test_timeout(self):
        """
        Test SREQ Timeouts
        """
        sreq = self.get_sreq()
        # client-side timeout
        start = time.time()
        # This is a try/except instead of an assertRaises because of a possible
        # subtle bug in zmq wherein a timeout=0 actually exceutes a single poll
        # before the timeout is reached.
        log.info("Sending tries=0, timeout=0")
        try:
            sreq.send("clear", "foo", tries=0, timeout=0)
        except salt.exceptions.SaltReqTimeoutError:
            pass
        assert time.time() - start < 1  # ensure we didn't wait

        # server-side timeout
        log.info("Sending tries=1, timeout=1")
        start = time.time()
        with self.assertRaises(salt.exceptions.SaltReqTimeoutError):
            sreq.send("clear", {"sleep": 2}, tries=1, timeout=1)
        assert time.time() - start >= 1  # ensure we actually tried once (1s)

        # server-side timeout with retries
        log.info("Sending tries=2, timeout=1")
        start = time.time()
        with self.assertRaises(salt.exceptions.SaltReqTimeoutError):
            sreq.send("clear", {"sleep": 2}, tries=2, timeout=1)
        assert time.time() - start >= 2  # ensure we actually tried twice (2s)

        # test a regular send afterwards (to make sure sockets aren't in a twist
        log.info("Sending regular send")
        assert sreq.send("clear", "foo") == {"enc": "clear", "load": "foo"}

    def test_destroy(self):
        """
        Test the __del__ capabilities
        """
        sreq = self.get_sreq()
        # ensure no exceptions when we go to destroy the sreq, since __del__
        # swallows exceptions, we have to call destroy directly
        sreq.destroy()

    def test_raw_vs_encoding_none(self):
        """
        Test that we handle the new raw parameter in 5.0.2 correctly based on
        encoding. When encoding is None loads should return bytes
        """
        payload = salt.payload.Serial("msgpack")
        dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
        idata = {dtvalue: "strval"}
        sdata = payload.dumps(idata.copy())
        odata = payload.loads(sdata, encoding=None)
        assert isinstance(odata[dtvalue], six.string_types)

    def test_raw_vs_encoding_utf8(self):
        """
        Test that we handle the new raw parameter in 5.0.2 correctly based on
        encoding. When encoding is utf-8 loads should return unicode
        """
        payload = salt.payload.Serial("msgpack")
        dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
        idata = {dtvalue: "strval"}
        sdata = payload.dumps(idata.copy())
        odata = payload.loads(sdata, encoding="utf-8")
        assert isinstance(odata[dtvalue], six.text_type)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.modules.deb_postgres as deb_postgres

# Import salt libs
from salt.ext import six

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import Mock, patch
from tests.support.unit import TestCase

LSCLUSTER = """\
8.4 main 5432 online postgres /srv/8.4/main \
        /var/log/postgresql/postgresql-8.4-main.log
9.1 main 5433 online postgres /srv/9.1/main \
        /var/log/postgresql/postgresql-9.1-main.log
"""


class PostgresClusterTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        self.cmd_run_all_mock = Mock(return_value={"stdout": LSCLUSTER})
        self.addCleanup(delattr, self, "cmd_run_all_mock")
        patcher = patch(
            "salt.utils.path.which", Mock(return_value="/usr/bin/pg_createcluster")
        )
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            deb_postgres: {
                "__salt__": {
                    "config.option": Mock(),
                    "cmd.run_all": self.cmd_run_all_mock,
                    "file.chown": Mock(),
                    "file.remove": Mock(),
                }
            }
        }

    def test_cluster_create(self):
        deb_postgres.cluster_create(
            "9.3",
            "main",
            port="5432",
            locale="fr_FR",
            encoding="UTF-8",
            datadir="/opt/postgresql",
        )
        cmdstr = (
            "/usr/bin/pg_createcluster "
            "--port 5432 --locale fr_FR --encoding UTF-8 "
            "--datadir /opt/postgresql "
            "9.3 main"
        )
        self.assertEqual(cmdstr, self.cmd_run_all_mock.call_args[0][0])

    def test_cluster_create_with_initdb_options(self):
        deb_postgres.cluster_create(
            "11",
            "main",
            port="5432",
            locale="fr_FR",
            encoding="UTF-8",
            datadir="/opt/postgresql",
            allow_group_access=True,
            data_checksums=True,
            wal_segsize="32",
        )
        cmdstr = (
            "/usr/bin/pg_createcluster "
            "--port 5432 --locale fr_FR --encoding UTF-8 "
            "--datadir /opt/postgresql "
            "11 main "
            "-- "
            "--allow-group-access "
            "--data-checksums "
            "--wal-segsize 32"
        )
        self.assertEqual(cmdstr, self.cmd_run_all_mock.call_args[0][0])

    # XXX version should be a string but from cmdline you get a float
    # def test_cluster_create_with_float(self):
    #     self.assertRaises(AssertionError, deb_postgres.cluster_create,
    #                       (9.3,'main',),
    #                       dict(port='5432',
    #                            locale='fr_FR',
    #                            encoding='UTF-8',
    #                            datadir='/opt/postgresql'))


class PostgresLsClusterTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        self.cmd_run_all_mock = Mock(return_value={"stdout": LSCLUSTER})
        self.addCleanup(delattr, self, "cmd_run_all_mock")
        patcher = patch(
            "salt.utils.path.which", Mock(return_value="/usr/bin/pg_lsclusters")
        )
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            deb_postgres: {
                "__salt__": {
                    "config.option": Mock(),
                    "cmd.run_all": self.cmd_run_all_mock,
                    "file.chown": Mock(),
                    "file.remove": Mock(),
                }
            }
        }

    def test_parse_pg_lsclusters(self):
        stdout = LSCLUSTER
        self.maxDiff = None
        self.assertDictEqual(
            {
                ("8.4/main"): {
                    "port": 5432,
                    "status": "online",
                    "user": "postgres",
                    "datadir": "/srv/8.4/main",
                    "log": "/var/log/postgresql/postgresql-8.4-main.log",
                },
                ("9.1/main"): {
                    "port": 5433,
                    "status": "online",
                    "user": "postgres",
                    "datadir": "/srv/9.1/main",
                    "log": "/var/log/postgresql/postgresql-9.1-main.log",
                },
            },
            deb_postgres._parse_pg_lscluster(stdout),
        )

    def test_cluster_list(self):
        return_list = deb_postgres.cluster_list()
        self.assertEqual(
            "/usr/bin/pg_lsclusters --no-header", self.cmd_run_all_mock.call_args[0][0]
        )
        if six.PY2:
            # Python 3 returns iterable views (dict_keys in this case) on
            # dict.keys() calls instead of lists. We should only perform
            # this check in Python 2.
            self.assertIsInstance(return_list, list)
        return_dict = deb_postgres.cluster_list(verbose=True)
        self.assertIsInstance(return_dict, dict)

    def test_cluster_exists(self):
        self.assertTrue(deb_postgres.cluster_exists("8.4") is True)
        self.assertTrue(deb_postgres.cluster_exists("8.4", "main") is True)
        self.assertFalse(deb_postgres.cluster_exists("3.4", "main"))


class PostgresDeleteClusterTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        self.cmd_run_all_mock = Mock(return_value={"stdout": LSCLUSTER})
        self.addCleanup(delattr, self, "cmd_run_all_mock")
        patcher = patch(
            "salt.utils.path.which", Mock(return_value="/usr/bin/pg_dropcluster")
        )
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            deb_postgres: {
                "__salt__": {
                    "config.option": Mock(),
                    "cmd.run_all": self.cmd_run_all_mock,
                    "file.chown": Mock(),
                    "file.remove": Mock(),
                }
            }
        }

    def test_cluster_delete(self):
        deb_postgres.cluster_remove("9.3", "main")
        self.assertEqual(
            "/usr/bin/pg_dropcluster 9.3 main", self.cmd_run_all_mock.call_args[0][0]
        )
        deb_postgres.cluster_remove("9.3", "main", stop=True)
        self.assertEqual(
            "/usr/bin/pg_dropcluster --stop 9.3 main",
            self.cmd_run_all_mock.call_args[0][0],
        )



# -*- coding: utf-8 -*-
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.grafana_datasource as grafana_datasource

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock, patch
from tests.support.unit import TestCase

profile = {
    "grafana_url": "http://grafana",
    "grafana_token": "token",
}


def mock_json_response(data):
    response = MagicMock()
    response.json = MagicMock(return_value=data)
    return Mock(return_value=response)


class GrafanaDatasourceTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {grafana_datasource: {}}

    def test_present(self):
        with patch("requests.get", mock_json_response([])):
            with patch("requests.post") as rpost:
                ret = grafana_datasource.present("test", "type", "url", profile=profile)
                rpost.assert_called_once_with(
                    "http://grafana/api/datasources",
                    grafana_datasource._get_json_data("test", "type", "url"),
                    headers={
                        "Authorization": "Bearer token",
                        "Accept": "application/json",
                    },
                    timeout=3,
                )
                self.assertTrue(ret["result"])
                self.assertEqual(ret["comment"], "New data source test added")

        data = grafana_datasource._get_json_data("test", "type", "url")
        data.update({"id": 1, "orgId": 1})
        with patch("requests.get", mock_json_response([data])):
            with patch("requests.put") as rput:
                ret = grafana_datasource.present("test", "type", "url", profile=profile)
                rput.assert_called_once_with(
                    "http://grafana/api/datasources/1",
                    grafana_datasource._get_json_data("test", "type", "url"),
                    headers={
                        "Authorization": "Bearer token",
                        "Accept": "application/json",
                    },
                    timeout=3,
                )
                self.assertTrue(ret["result"])
                self.assertEqual(ret["comment"], "Data source test already up-to-date")
                self.assertEqual(ret["changes"], {})

            with patch("requests.put") as rput:
                ret = grafana_datasource.present(
                    "test", "type", "newurl", profile=profile
                )
                rput.assert_called_once_with(
                    "http://grafana/api/datasources/1",
                    grafana_datasource._get_json_data("test", "type", "newurl"),
                    headers={
                        "Authorization": "Bearer token",
                        "Accept": "application/json",
                    },
                    timeout=3,
                )
                self.assertTrue(ret["result"])
                self.assertEqual(ret["comment"], "Data source test updated")
                self.assertEqual(
                    ret["changes"], {"old": {"url": "url"}, "new": {"url": "newurl"}}
                )

    def test_absent(self):
        with patch("requests.get", mock_json_response([])):
            with patch("requests.delete") as rdelete:
                ret = grafana_datasource.absent("test", profile=profile)
                self.assertTrue(rdelete.call_count == 0)
                self.assertTrue(ret["result"])
                self.assertEqual(ret["comment"], "Data source test already absent")

        with patch("requests.get", mock_json_response([{"name": "test", "id": 1}])):
            with patch("requests.delete") as rdelete:
                ret = grafana_datasource.absent("test", profile=profile)
                rdelete.assert_called_once_with(
                    "http://grafana/api/datasources/1",
                    headers={
                        "Authorization": "Bearer token",
                        "Accept": "application/json",
                    },
                    timeout=3,
                )
                self.assertTrue(ret["result"])
                self.assertEqual(ret["comment"], "Data source test was deleted")



# -*- coding: utf-8 -*-
"""
    :codeauthor: Alexander Pyatkin <asp@thexyz.net>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.bower as bower
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BowerTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.bower
    """

    def setup_loader_modules(self):
        return {bower: {"_check_valid_version": MagicMock(return_value=True)}}

    def test_install_with_error(self):
        """
        Test if it raises an exception when install package fails
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertRaises(
                CommandExecutionError, bower.install, "/path/to/project", "underscore"
            )

    def test_install_new_package(self):
        """
        Test if it returns True when install package succeeds
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": '{"underscore":{}}'})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(bower.install("/path/to/project", "underscore"))

    def test_install_existing_package(self):
        """
        Test if it returns False when package already installed
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": "{}"})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertFalse(bower.install("/path/to/project", "underscore"))

    def test_uninstall_with_error(self):
        """
        Test if it raises an exception when uninstall package fails
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertRaises(
                CommandExecutionError, bower.uninstall, "/path/to/project", "underscore"
            )

    def test_uninstall_existing_package(self):
        """
        Test if it returns True when uninstall package succeeds
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": '{"underscore": {}}'})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(bower.uninstall("/path/to/project", "underscore"))

    def test_uninstall_missing_package(self):
        """
        Test if it returns False when package is not installed
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": "{}"})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertFalse(bower.uninstall("/path/to/project", "underscore"))

    def test_list_packages_with_error(self):
        """
        Test if it raises an exception when list installed packages fails
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertRaises(CommandExecutionError, bower.list_, "/path/to/project")

    def test_list_packages_success(self):
        """
        Test if it lists installed Bower packages
        """
        output = '{"dependencies": {"underscore": {}, "jquery":{}}}'
        mock = MagicMock(return_value={"retcode": 0, "stdout": output})
        with patch.dict(bower.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(
                bower.list_("/path/to/project"), {"underscore": {}, "jquery": {}}
            )



# -*- coding: utf-8 -*-
"""
    Salt proxy module

    .. versionadded:: 2015.8.3

    Module to deploy and manage salt-proxy processes
    on a minion.
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os

# Import 3rd-party libs
import salt.ext.six.moves
import salt.syspaths

# Import Salt libs
import salt.utils.files

log = logging.getLogger(__name__)


def _write_proxy_conf(proxyfile):
    """
    write to file
    """
    msg = "Invalid value for proxy file provided!, Supplied value = {0}".format(
        proxyfile
    )

    log.trace("Salt Proxy Module: write proxy conf")

    if proxyfile:
        log.debug("Writing proxy conf file")
        with salt.utils.files.fopen(proxyfile, "w") as proxy_conf:
            proxy_conf.write(
                salt.utils.stringutils.to_str(
                    "master: {0}".format(__grains__["master"])
                )
            )
        msg = "Wrote proxy file {0}".format(proxyfile)
        log.debug(msg)

    return msg


def _proxy_conf_file(proxyfile, test):
    """
    Check if proxy conf exists and update
    """
    changes_old = []
    changes_new = []
    success = True
    if not os.path.exists(proxyfile):
        try:
            if not test:
                changes_new.append(_write_proxy_conf(proxyfile))
                msg = "Salt Proxy: Wrote proxy conf {0}".format(proxyfile)
            else:
                msg = "Salt Proxy: Update required to proxy conf {0}".format(proxyfile)
        except (OSError, IOError) as err:
            success = False
            msg = "Salt Proxy: Error writing proxy file {0}".format(err)
            log.error(msg)
            changes_new.append(msg)
        changes_new.append(msg)
        log.debug(msg)
    else:
        msg = "Salt Proxy: {0} already exists, skipping".format(proxyfile)
        changes_old.append(msg)
        log.debug(msg)
    return success, changes_new, changes_old


def _is_proxy_running(proxyname):
    """
    Check if proxy for this name is running
    """
    cmd = 'ps ax | grep "salt-proxy --proxyid={0}" | grep -v grep'.format(
        salt.ext.six.moves.shlex_quote(proxyname)
    )
    cmdout = __salt__["cmd.run_all"](cmd, timeout=5, python_shell=True)
    if not cmdout["stdout"]:
        return False
    else:
        return True


def _proxy_process(proxyname, test):
    """
    Check and execute proxy process
    """
    changes_old = []
    changes_new = []
    if not _is_proxy_running(proxyname):
        if not test:
            __salt__["cmd.run_all"](
                "salt-proxy --proxyid={0} -l info -d".format(
                    salt.ext.six.moves.shlex_quote(proxyname)
                ),
                timeout=5,
            )
            changes_new.append(
                "Salt Proxy: Started proxy process for {0}".format(proxyname)
            )
        else:
            changes_new.append(
                "Salt Proxy: process {0} will be started".format(proxyname)
            )
    else:
        changes_old.append("Salt Proxy: already running for {0}".format(proxyname))
    return True, changes_new, changes_old


def configure_proxy(proxyname, start=True):
    """
    Create the salt proxy file and start the proxy process
    if required

    Parameters:
        proxyname:
            Name to be used for this proxy (should match entries in pillar)
        start:
            Boolean indicating if the process should be started
            default = True

    CLI Example:

    .. code-block:: bash

        salt deviceminion salt_proxy.configure_proxy p8000
    """
    changes_new = []
    changes_old = []
    status_file = True
    test = __opts__["test"]

    # write the proxy file if necessary
    proxyfile = os.path.join(salt.syspaths.CONFIG_DIR, "proxy")
    status_file, msg_new, msg_old = _proxy_conf_file(proxyfile, test)
    changes_new.extend(msg_new)
    changes_old.extend(msg_old)
    status_proc = False

    # start the proxy process
    if start:
        status_proc, msg_new, msg_old = _proxy_process(proxyname, test)
        changes_old.extend(msg_old)
        changes_new.extend(msg_new)
    else:
        changes_old.append("Start is False, not starting salt-proxy process")
        log.debug("Process not started")

    return {
        "result": status_file and status_proc,
        "changes": {"old": "\n".join(changes_old), "new": "\n".join(changes_new)},
    }


def is_running(proxyname):
    """
    Check if the salt-proxy process associated
    with this proxy (name) is running.

    Returns True if the process is running
    False otherwise

    Parameters:
        proxyname:
            String name of the proxy (p8000 for example)

    CLI Example:

    .. code-block:: bash

        salt deviceminion salt_proxy.is_running p8000
    """
    return {"result": _is_proxy_running(proxyname)}



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import textwrap

import salt.modules.npm as npm

# Import Salt Libs
import salt.utils.json
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class NpmTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.npm
    """

    def setup_loader_modules(self):
        patcher = patch(
            "salt.modules.npm._check_valid_version", MagicMock(return_value=True)
        )
        patcher.start()
        self.addCleanup(patcher.stop)
        return {npm: {}}

    # 'install' function tests: 4

    def test_install(self):
        """
        Test if it installs an NPM package.
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertRaises(CommandExecutionError, npm.install, "coffee-script")

        # This is at least somewhat closer to the actual output format.
        mock_json_out = textwrap.dedent(
            """\
        [
          {
            "salt": "SALT"
          }
        ]"""
        )

        # Successful run, expected output format
        mock = MagicMock(
            return_value={"retcode": 0, "stderr": "", "stdout": mock_json_out}
        )
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(npm.install("coffee-script"), [{"salt": "SALT"}])

        mock_json_out_extra = textwrap.dedent(
            """\
        Compilation output here

        [bcrypt] Success: "/tmp/node_modules/bcrypt/foo" is installed via remote"
        [grpc] Success: "/usr/lib/node_modules/@foo/bar" is installed via remote"
        [
           {
              "from" : "express@",
              "name" : "express",
              "dependencies" : {
                 "escape-html" : {
                    "from" : "escape-html@~1.0.3",
                    "dependencies" : {},
                    "version" : "1.0.3"
                 }
              },
              "version" : "4.16.3"
           }
        ]"""
        )
        extra_expected = [
            {
                "dependencies": {
                    "escape-html": {
                        "dependencies": {},
                        "from": "escape-html@~1.0.3",
                        "version": "1.0.3",
                    }
                },
                "from": "express@",
                "name": "express",
                "version": "4.16.3",
            }
        ]

        # Successful run, expected output format with additional leading text
        mock = MagicMock(
            return_value={"retcode": 0, "stderr": "", "stdout": mock_json_out_extra}
        )
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(npm.install("coffee-script"), extra_expected)

        # Successful run, unexpected output format
        mock = MagicMock(return_value={"retcode": 0, "stderr": "", "stdout": "SALT"})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            mock_err = MagicMock(side_effect=ValueError())
            # When JSON isn't successfully parsed, return should equal input
            with patch.object(salt.utils.json, "loads", mock_err):
                self.assertEqual(npm.install("coffee-script"), "SALT")

    # 'uninstall' function tests: 1

    def test_uninstall(self):
        """
        Test if it uninstalls an NPM package.
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertFalse(npm.uninstall("coffee-script"))

        mock = MagicMock(return_value={"retcode": 0, "stderr": ""})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(npm.uninstall("coffee-script"))

    # 'list_' function tests: 1

    def test_list(self):
        """
        Test if it list installed NPM packages.
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertRaises(CommandExecutionError, npm.list_, "coffee-script")

        mock = MagicMock(
            return_value={
                "retcode": 0,
                "stderr": "error",
                "stdout": '{"salt": ["SALT"]}',
            }
        )
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            mock_err = MagicMock(return_value={"dependencies": "SALT"})
            with patch.object(salt.utils.json, "loads", mock_err):
                self.assertEqual(npm.list_("coffee-script"), "SALT")

    # 'cache_clean' function tests: 1

    def test_cache_clean(self):
        """
        Test if it cleans the cached NPM packages.
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertFalse(npm.cache_clean())

        mock = MagicMock(return_value={"retcode": 0})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(npm.cache_clean())

        mock = MagicMock(return_value={"retcode": 0})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(npm.cache_clean("coffee-script"))

    # 'cache_list' function tests: 1

    def test_cache_list(self):
        """
        Test if it lists the NPM cache.
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertRaises(CommandExecutionError, npm.cache_list)

        mock = MagicMock(
            return_value={"retcode": 0, "stderr": "error", "stdout": ["~/.npm"]}
        )
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(npm.cache_list(), ["~/.npm"])

        mock = MagicMock(return_value={"retcode": 0, "stderr": "error", "stdout": ""})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(npm.cache_list("coffee-script"), "")

    # 'cache_path' function tests: 1

    def test_cache_path(self):
        """
        Test if it prints the NPM cache path.
        """
        mock = MagicMock(return_value={"retcode": 1, "stderr": "error"})
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(npm.cache_path(), "error")

        mock = MagicMock(
            return_value={"retcode": 0, "stderr": "error", "stdout": "/User/salt/.npm"}
        )
        with patch.dict(npm.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(npm.cache_path(), "/User/salt/.npm")



# -*- coding: utf-8 -*-
"""
This module contains the function calls to execute command line scripts
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import functools
import logging
import os
import signal
import sys
import threading
import time
import traceback
from random import randint

import salt.defaults.exitcodes  # pylint: disable=unused-import
import salt.ext.six as six

# Import salt libs
from salt.exceptions import SaltClientError, SaltReqTimeoutError, SaltSystemExit

log = logging.getLogger(__name__)


def _handle_interrupt(exc, original_exc, hardfail=False, trace=""):
    """
    if hardfailing:
        If we got the original stacktrace, log it
        If all cases, raise the original exception
        but this is logically part the initial
        stack.
    else just let salt exit gracefully

    """
    if hardfail:
        if trace:
            log.error(trace)
        raise original_exc
    else:
        raise exc


def _handle_signals(client, signum, sigframe):
    try:
        # This raises AttributeError on Python 3.4 and 3.5 if there is no current exception.
        # Ref: https://bugs.python.org/issue23003
        trace = traceback.format_exc()
    except AttributeError:
        trace = ""
    try:
        hardcrash = client.options.hard_crash
    except (AttributeError, KeyError):
        hardcrash = False

    if signum == signal.SIGINT:
        exit_msg = "\nExiting gracefully on Ctrl-c"
        try:
            jid = client.local_client.pub_data["jid"]
            exit_msg += (
                "\n"
                "This job's jid is: {0}\n"
                "The minions may not have all finished running and any remaining "
                "minions will return upon completion. To look up the return data "
                "for this job later, run the following command:\n\n"
                "salt-run jobs.lookup_jid {0}".format(jid)
            )
        except (AttributeError, KeyError):
            pass
    else:
        exit_msg = None

    _handle_interrupt(
        SystemExit(exit_msg),
        Exception("\nExiting with hard crash on Ctrl-c"),
        hardcrash,
        trace=trace,
    )


def _install_signal_handlers(client):
    # Install the SIGINT/SIGTERM handlers if not done so far
    if signal.getsignal(signal.SIGINT) is signal.SIG_DFL:
        # No custom signal handling was added, install our own
        signal.signal(signal.SIGINT, functools.partial(_handle_signals, client))

    if signal.getsignal(signal.SIGTERM) is signal.SIG_DFL:
        # No custom signal handling was added, install our own
        signal.signal(signal.SIGINT, functools.partial(_handle_signals, client))


def salt_master():
    """
    Start the salt master.
    """
    import salt.cli.daemons

    # Fix for setuptools generated scripts, so that it will
    # work with multiprocessing fork emulation.
    # (see multiprocessing.forking.get_preparation_data())
    if __name__ != "__main__":
        sys.modules["__main__"] = sys.modules[__name__]

    # REMOVEME after Python 2.7 support is dropped (also the six import)
    if six.PY2:
        from salt.utils.versions import warn_until

        # Message borrowed from pip's deprecation warning
        warn_until(
            "Sodium",
            "Python 2.7 will reach the end of its life on January 1st,"
            " 2020. Please upgrade your Python as Python 2.7 won't be"
            " maintained after that date.  Salt will drop support for"
            " Python 2.7 in the 3001 release or later.",
        )
    # END REMOVEME
    master = salt.cli.daemons.Master()
    master.start()


def minion_process():
    """
    Start a minion process
    """
    import salt.utils.platform
    import salt.utils.process
    import salt.cli.daemons

    # salt_minion spawns this function in a new process

    salt.utils.process.appendproctitle("KeepAlive")

    def handle_hup(manager, sig, frame):
        manager.minion.reload()

    lock = threading.RLock()

    def suicide_when_without_parent(parent_pid):
        """
        Have the minion suicide if the parent process is gone

        NOTE: small race issue where the parent PID could be replace
        with another process with same PID!
        """
        while lock.acquire(blocking=False):
            lock.release()
            time.sleep(5)
            try:
                # check pid alive (Unix only trick!)
                if os.getuid() == 0 and not salt.utils.platform.is_windows():
                    os.kill(parent_pid, 0)
            except OSError as exc:
                # forcibly exit, regular sys.exit raises an exception-- which
                # isn't sufficient in a thread
                log.error("Minion process encountered exception: %s", exc)
                os._exit(salt.defaults.exitcodes.EX_GENERIC)

    try:
        if not salt.utils.platform.is_windows():
            thread = threading.Thread(
                target=suicide_when_without_parent, args=(os.getppid(),)
            )
            thread.start()

        minion = salt.cli.daemons.Minion()
        signal.signal(signal.SIGHUP, functools.partial(handle_hup, minion))
        minion.start()
    except (SaltClientError, SaltReqTimeoutError, SaltSystemExit) as exc:
        lock.acquire(blocking=True)
        log.warning(
            "Fatal functionality error caught by minion handler:\n", exc_info=True
        )
        log.warning("** Restarting minion **")
        delay = 60
        if minion is not None and hasattr(minion, "config"):
            delay = minion.config.get("random_reauth_delay", 60)
        delay = randint(1, delay)
        log.info("waiting random_reauth_delay %ss", delay)
        time.sleep(delay)
        sys.exit(salt.defaults.exitcodes.SALT_KEEPALIVE)
    finally:
        lock.acquire(blocking=True)


def salt_minion():
    """
    Start the salt minion in a subprocess.
    Auto restart minion on error.
    """
    import signal

    import salt.utils.platform
    import salt.utils.process

    salt.utils.process.notify_systemd()

    import salt.cli.daemons
    import multiprocessing

    # Fix for setuptools generated scripts, so that it will
    # work with multiprocessing fork emulation.
    # (see multiprocessing.forking.get_preparation_data())
    if __name__ != "__main__":
        sys.modules["__main__"] = sys.modules[__name__]

    if "" in sys.path:
        sys.path.remove("")

    if salt.utils.platform.is_windows():
        minion = salt.cli.daemons.Minion()
        minion.start()
        return
    # REMOVEME after Python 2.7 support is dropped (also the six import)
    elif six.PY2:
        from salt.utils.versions import warn_until

        # Message borrowed from pip's deprecation warning
        warn_until(
            "Sodium",
            "Python 2.7 will reach the end of its life on January 1st,"
            " 2020. Please upgrade your Python as Python 2.7 won't be"
            " maintained after that date.  Salt will drop support for"
            " Python 2.7 in the 3001 release or later.",
        )
    # END REMOVEME

    if "--disable-keepalive" in sys.argv:
        sys.argv.remove("--disable-keepalive")
        minion = salt.cli.daemons.Minion()
        minion.start()
        return

    def escalate_signal_to_process(
        pid, signum, sigframe
    ):  # pylint: disable=unused-argument
        """
        Escalate the signal received to the multiprocessing process that
        is actually running the minion
        """
        # escalate signal
        os.kill(pid, signum)

    # keep one minion subprocess running
    prev_sigint_handler = signal.getsignal(signal.SIGINT)
    prev_sigterm_handler = signal.getsignal(signal.SIGTERM)
    while True:
        try:
            process = multiprocessing.Process(target=minion_process)
            process.start()
            signal.signal(
                signal.SIGTERM,
                functools.partial(escalate_signal_to_process, process.pid),
            )
            signal.signal(
                signal.SIGINT,
                functools.partial(escalate_signal_to_process, process.pid),
            )
            signal.signal(
                signal.SIGHUP,
                functools.partial(escalate_signal_to_process, process.pid),
            )
        except Exception:  # pylint: disable=broad-except
            # if multiprocessing does not work
            minion = salt.cli.daemons.Minion()
            minion.start()
            break

        process.join()

        # Process exited or was terminated. Since we're going to try to restart
        # it, we MUST, reset signal handling to the previous handlers
        signal.signal(signal.SIGINT, prev_sigint_handler)
        signal.signal(signal.SIGTERM, prev_sigterm_handler)

        if not process.exitcode == salt.defaults.exitcodes.SALT_KEEPALIVE:
            sys.exit(process.exitcode)
        # ontop of the random_reauth_delay already preformed
        # delay extra to reduce flooding and free resources
        # NOTE: values are static but should be fine.
        time.sleep(2 + randint(1, 10))
        # need to reset logging because new minion objects
        # cause extra log handlers to accumulate
        rlogger = logging.getLogger()
        for handler in rlogger.handlers:
            rlogger.removeHandler(handler)
        logging.basicConfig()


def proxy_minion_process(queue):
    """
    Start a proxy minion process
    """
    import salt.cli.daemons
    import salt.utils.platform

    # salt_minion spawns this function in a new process

    lock = threading.RLock()

    def suicide_when_without_parent(parent_pid):
        """
        Have the minion suicide if the parent process is gone

        NOTE: there is a small race issue where the parent PID could be replace
        with another process with the same PID!
        """
        while lock.acquire(blocking=False):
            lock.release()
            time.sleep(5)
            try:
                # check pid alive (Unix only trick!)
                os.kill(parent_pid, 0)
            except OSError:
                # forcibly exit, regular sys.exit raises an exception-- which
                # isn't sufficient in a thread
                os._exit(999)

    try:
        if not salt.utils.platform.is_windows():
            thread = threading.Thread(
                target=suicide_when_without_parent, args=(os.getppid(),)
            )
            thread.start()

        restart = False
        proxyminion = None
        status = salt.defaults.exitcodes.EX_OK
        proxyminion = salt.cli.daemons.ProxyMinion()
        proxyminion.start()
        # pylint: disable=broad-except
    except (Exception, SaltClientError, SaltReqTimeoutError, SaltSystemExit,) as exc:
        # pylint: enable=broad-except
        log.error("Proxy Minion failed to start: ", exc_info=True)
        restart = True
        # status is superfluous since the process will be restarted
        status = salt.defaults.exitcodes.SALT_KEEPALIVE
    except SystemExit as exc:
        restart = False
        status = exc.code
    finally:
        lock.acquire(blocking=True)

    if restart is True:
        log.warning("** Restarting proxy minion **")
        delay = 60
        if proxyminion is not None:
            if hasattr(proxyminion, "config"):
                delay = proxyminion.config.get("random_reauth_delay", 60)
        random_delay = randint(1, delay)
        log.info("Sleeping random_reauth_delay of %s seconds", random_delay)
        # preform delay after minion resources have been cleaned
        queue.put(random_delay)
    else:
        queue.put(0)
    sys.exit(status)


def salt_proxy():
    """
    Start a proxy minion.
    """
    import salt.cli.daemons
    import salt.utils.platform
    import multiprocessing

    if "" in sys.path:
        sys.path.remove("")

    if salt.utils.platform.is_windows():
        proxyminion = salt.cli.daemons.ProxyMinion()
        proxyminion.start()
        return

    if "--disable-keepalive" in sys.argv:
        sys.argv.remove("--disable-keepalive")
        proxyminion = salt.cli.daemons.ProxyMinion()
        proxyminion.start()
        return

    # keep one minion subprocess running
    while True:
        try:
            queue = multiprocessing.Queue()
        except Exception:  # pylint: disable=broad-except
            # This breaks in containers
            proxyminion = salt.cli.daemons.ProxyMinion()
            proxyminion.start()
            return
        process = multiprocessing.Process(target=proxy_minion_process, args=(queue,))
        process.start()
        try:
            process.join()
            try:
                restart_delay = queue.get(block=False)
            except Exception:  # pylint: disable=broad-except
                if process.exitcode == 0:
                    # Minion process ended naturally, Ctrl+C or --version
                    break
                restart_delay = 60
            if restart_delay == 0:
                # Minion process ended naturally, Ctrl+C, --version, etc.
                sys.exit(process.exitcode)
            # delay restart to reduce flooding and allow network resources to close
            time.sleep(restart_delay)
        except KeyboardInterrupt:
            break
        # need to reset logging because new minion objects
        # cause extra log handlers to accumulate
        rlogger = logging.getLogger()
        for handler in rlogger.handlers:
            rlogger.removeHandler(handler)
        logging.basicConfig()


def salt_syndic():
    """
    Start the salt syndic.
    """
    import salt.utils.process

    salt.utils.process.notify_systemd()

    import salt.cli.daemons

    pid = os.getpid()
    try:
        syndic = salt.cli.daemons.Syndic()
        syndic.start()
    except KeyboardInterrupt:
        os.kill(pid, 15)


def salt_key():
    """
    Manage the authentication keys with salt-key.
    """
    import salt.cli.key

    try:
        client = salt.cli.key.SaltKey()
        _install_signal_handlers(client)
        client.run()
    except Exception as err:  # pylint: disable=broad-except
        sys.stderr.write("Error: {0}\n".format(err))


def salt_cp():
    """
    Publish commands to the salt system from the command line on the
    master.
    """
    import salt.cli.cp

    client = salt.cli.cp.SaltCPCli()
    _install_signal_handlers(client)
    client.run()


def salt_call():
    """
    Directly call a salt command in the modules, does not require a running
    salt minion to run.
    """
    import salt.cli.call

    if "" in sys.path:
        sys.path.remove("")
    client = salt.cli.call.SaltCall()
    _install_signal_handlers(client)
    client.run()


def salt_run():
    """
    Execute a salt convenience routine.
    """
    import salt.cli.run

    if "" in sys.path:
        sys.path.remove("")
    client = salt.cli.run.SaltRun()
    _install_signal_handlers(client)
    client.run()


def salt_ssh():
    """
    Execute the salt-ssh system
    """
    import salt.cli.ssh

    if "" in sys.path:
        sys.path.remove("")
    try:
        client = salt.cli.ssh.SaltSSH()
        _install_signal_handlers(client)
        client.run()
    except SaltClientError as err:
        trace = traceback.format_exc()
        try:
            hardcrash = client.options.hard_crash
        except (AttributeError, KeyError):
            hardcrash = False
        _handle_interrupt(SystemExit(err), err, hardcrash, trace=trace)


def salt_cloud():
    """
    The main function for salt-cloud
    """
    # Define 'salt' global so we may use it after ImportError. Otherwise,
    # UnboundLocalError will be raised.
    global salt  # pylint: disable=W0602

    try:
        # Late-imports for CLI performance
        import salt.cloud
        import salt.cloud.cli
    except ImportError as e:
        # No salt cloud on Windows
        log.error("Error importing salt cloud: %s", e)
        print("salt-cloud is not available in this system")
        sys.exit(salt.defaults.exitcodes.EX_UNAVAILABLE)
    if "" in sys.path:
        sys.path.remove("")

    client = salt.cloud.cli.SaltCloud()
    _install_signal_handlers(client)
    client.run()


def salt_api():
    """
    The main function for salt-api
    """
    import salt.utils.process

    salt.utils.process.notify_systemd()

    import salt.cli.api

    sapi = salt.cli.api.SaltAPI()  # pylint: disable=E1120
    sapi.start()


def salt_main():
    """
    Publish commands to the salt system from the command line on the
    master.
    """
    import salt.cli.salt

    if "" in sys.path:
        sys.path.remove("")
    client = salt.cli.salt.SaltCMD()
    _install_signal_handlers(client)
    client.run()


def salt_spm():
    """
    The main function for spm, the Salt Package Manager

    .. versionadded:: 2015.8.0
    """
    import salt.cli.spm

    spm = salt.cli.spm.SPM()  # pylint: disable=E1120
    spm.run()


def salt_extend(extension, name, description, salt_dir, merge):
    """
    Quickstart for developing on the saltstack installation

    .. versionadded:: 2016.11.0
    """
    import salt.utils.extend

    salt.utils.extend.run(
        extension=extension,
        name=name,
        description=description,
        salt_dir=salt_dir,
        merge=merge,
    )


def salt_unity():
    """
    Change the args and redirect to another salt script
    """
    avail = []
    for fun in dir(sys.modules[__name__]):
        if fun.startswith("salt"):
            avail.append(fun[5:])
    if len(sys.argv) < 2:
        msg = "Must pass in a salt command, available commands are:"
        for cmd in avail:
            msg += "\n{0}".format(cmd)
        print(msg)
        sys.exit(1)
    cmd = sys.argv[1]
    if cmd not in avail:
        # Fall back to the salt command
        sys.argv[0] = "salt"
        s_fun = salt_main
    else:
        sys.argv[0] = "salt-{0}".format(cmd)
        sys.argv.pop(1)
        s_fun = getattr(sys.modules[__name__], "salt_{0}".format(cmd))
    s_fun()



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.modules.qemu_img as qemu_img

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class QemuimgTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.qemu_img
    """

    def setup_loader_modules(self):
        return {qemu_img: {}}

    def test_make_image(self):
        """
        Test for create a blank virtual machine image file
        of the specified size in megabytes
        """
        with patch.object(
            os.path, "isabs", MagicMock(side_effect=[False, True, True, True])
        ):
            self.assertEqual(qemu_img.make_image("location", "size", "fmt"), "")

            with patch.object(
                os.path, "isdir", MagicMock(side_effect=[False, True, True])
            ):
                self.assertEqual(qemu_img.make_image("location", "size", "fmt"), "")

                with patch.dict(
                    qemu_img.__salt__,
                    {"cmd.retcode": MagicMock(side_effect=[False, True])},
                ):
                    self.assertEqual(
                        qemu_img.make_image("location", "size", "fmt"), "location"
                    )
                    self.assertEqual(qemu_img.make_image("location", "size", "fmt"), "")



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import sys

# Import Salt Libs
import salt.modules.pam as pam
from tests.support.mock import mock_open, patch

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf

MOCK_FILE = "ok ok ignore "


@skipIf(sys.platform.startswith("openbsd"), "OpenBSD does not use PAM")
class PamTestCase(TestCase):
    """
    Test cases for salt.modules.pam
    """

    # 'read_file' function tests: 1

    def test_read_file(self):
        """
        Test if the parsing function works
        """
        with patch("os.path.exists", return_value=True), patch(
            "salt.utils.files.fopen", mock_open(read_data=MOCK_FILE)
        ):
            self.assertListEqual(
                pam.read_file("/etc/pam.d/login"),
                [
                    {
                        "arguments": [],
                        "control_flag": "ok",
                        "interface": "ok",
                        "module": "ignore",
                    }
                ],
            )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.debconfmod as debconfmod

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class DebconfmodTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.debconfmod
    """

    def setup_loader_modules(self):
        return {debconfmod: {}}

    # 'set_file' function tests: 1

    def test_set_file(self):
        """
        Test to set debconf selections from a file or a template
        """
        name = "nullmailer"
        source = "salt://pathto/pkg.selections"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        comt = "Context must be formed as a dict"
        ret.update({"comment": comt})
        self.assertDictEqual(debconfmod.set_file(name, source, context="salt"), ret)

        comt = "Defaults must be formed as a dict"
        ret.update({"comment": comt})
        self.assertDictEqual(debconfmod.set_file(name, source, defaults="salt"), ret)

        with patch.dict(debconfmod.__opts__, {"test": True}):
            comt = "Debconf selections would have been set."
            ret.update({"comment": comt, "result": None})
            self.assertDictEqual(debconfmod.set_file(name, source), ret)

            with patch.dict(debconfmod.__opts__, {"test": False}):
                mock = MagicMock(return_value=True)
                with patch.dict(debconfmod.__salt__, {"debconf.set_file": mock}):
                    comt = "Debconf selections were set."
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(debconfmod.set_file(name, source), ret)

    # 'set' function tests: 1

    def test_set(self):
        """
        Test to set debconf selections
        """
        name = "nullmailer"
        data = {
            "shared/mailname": {"type": "string", "value": "server.domain.tld"},
            "nullmailer/relayhost": {"type": "string", "value": "mail.domain.tld"},
        }

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        changes = {
            "nullmailer/relayhost": "New value: mail.domain.tld",
            "shared/mailname": "New value: server.domain.tld",
        }

        mock = MagicMock(return_value=None)
        with patch.dict(debconfmod.__salt__, {"debconf.show": mock}):
            with patch.dict(debconfmod.__opts__, {"test": True}):
                ret.update({"changes": changes})
                self.assertDictEqual(debconfmod.set(name, data), ret)



# -*- coding: utf-8 -*-
#
# Author: Bo Maryniuk <bo@suse.de>
#
# Copyright 2017 SUSE LLC
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Ansible Support
===============

This module can have an optional minion-level
configuration in /etc/salt/minion.d/ as follows:

  ansible_timeout: 1200

The timeout is how many seconds Salt should wait for
any Ansible module to respond.
"""

from __future__ import absolute_import, print_function, unicode_literals

import fnmatch
import importlib
import json
import logging
import os
import subprocess
import sys

import salt.utils.decorators.path
import salt.utils.json
import salt.utils.platform
import salt.utils.stringutils
import salt.utils.timed_subprocess
import salt.utils.yaml
from salt.exceptions import CommandExecutionError, LoaderError
from salt.ext import six
from salt.utils.decorators import depends

try:
    import ansible
    import ansible.constants  # pylint: disable=no-name-in-module
    import ansible.modules  # pylint: disable=no-name-in-module
except ImportError:
    ansible = None

# Function alias to make sure not to shadow built-in's
__func_alias__ = {"list_": "list"}

__virtualname__ = "ansible"

log = logging.getLogger(__name__)


class AnsibleModuleResolver(object):
    """
    This class is to resolve all available modules in Ansible.
    """

    def __init__(self, opts):
        self.opts = opts
        self._modules_map = {}

    def _get_modules_map(self, path=None):
        """
        Get installed Ansible modules
        :return:
        """
        paths = {}
        root = ansible.modules.__path__[0]
        if not path:
            path = root
        for p_el in os.listdir(path):
            p_el_path = os.path.join(path, p_el)
            if os.path.islink(p_el_path):
                continue
            if os.path.isdir(p_el_path):
                paths.update(self._get_modules_map(p_el_path))
            else:
                if (
                    any(p_el.startswith(elm) for elm in ["__", "."])
                    or not p_el.endswith(".py")
                    or p_el in ansible.constants.IGNORE_FILES
                ):
                    continue
                p_el_path = p_el_path.replace(root, "").split(".")[0]
                als_name = (
                    p_el_path.replace(".", "").replace("/", "", 1).replace("/", ".")
                )
                paths[als_name] = p_el_path

        return paths

    def load_module(self, module):
        """
        Introspect Ansible module.

        :param module:
        :return:
        """
        m_ref = self._modules_map.get(module)
        if m_ref is None:
            raise LoaderError('Module "{0}" was not found'.format(module))
        mod = importlib.import_module(
            "ansible.modules{0}".format(
                ".".join([elm.split(".")[0] for elm in m_ref.split(os.path.sep)])
            )
        )

        return mod

    def get_modules_list(self, pattern=None):
        """
        Return module map references.
        :return:
        """
        if pattern and "*" not in pattern:
            pattern = "*{0}*".format(pattern)
        modules = []
        for m_name, m_path in self._modules_map.items():
            m_path = m_path.split(".")[0]
            m_name = ".".join([elm for elm in m_path.split(os.path.sep) if elm])
            if pattern and fnmatch.fnmatch(m_name, pattern) or not pattern:
                modules.append(m_name)
        return sorted(modules)

    def resolve(self):
        log.debug("Resolving Ansible modules")
        self._modules_map = self._get_modules_map()
        return self

    def install(self):
        log.debug("Installing Ansible modules")
        return self


class AnsibleModuleCaller(object):
    DEFAULT_TIMEOUT = 1200  # seconds (20 minutes)
    OPT_TIMEOUT_KEY = "ansible_timeout"

    def __init__(self, resolver):
        self._resolver = resolver
        self.timeout = self._resolver.opts.get(
            self.OPT_TIMEOUT_KEY, self.DEFAULT_TIMEOUT
        )

    def call(self, module, *args, **kwargs):
        """
        Call an Ansible module by invoking it.
        :param module: the name of the module.
        :param args: Arguments to the module
        :param kwargs: keywords to the module
        :return:
        """

        module = self._resolver.load_module(module)
        if not hasattr(module, "main"):
            raise CommandExecutionError(
                "This module is not callable "
                '(see "ansible.help {0}")'.format(
                    module.__name__.replace("ansible.modules.", "")
                )
            )
        if args:
            kwargs["_raw_params"] = " ".join(args)
        js_args = str(
            '{{"ANSIBLE_MODULE_ARGS": {args}}}'
        )  # future lint: disable=blacklisted-function
        js_args = js_args.format(args=salt.utils.json.dumps(kwargs))

        proc_out = salt.utils.timed_subprocess.TimedProc(
            ["echo", "{0}".format(js_args)],
            stdout=subprocess.PIPE,
            timeout=self.timeout,
        )
        proc_out.run()
        proc_out_stdout = salt.utils.stringutils.to_str(proc_out.stdout)
        proc_exc = salt.utils.timed_subprocess.TimedProc(
            [sys.executable, module.__file__],
            stdin=proc_out_stdout,
            stdout=subprocess.PIPE,
            timeout=self.timeout,
        )
        proc_exc.run()

        try:
            out = salt.utils.json.loads(proc_exc.stdout)
        except ValueError as ex:
            out = {
                "Error": (
                    proc_exc.stderr and (proc_exc.stderr + ".") or six.text_type(ex)
                )
            }
            if proc_exc.stdout:
                out["Given JSON output"] = proc_exc.stdout
            return out

        if "invocation" in out:
            del out["invocation"]

        out["timeout"] = self.timeout

        return out


_resolver = None
_caller = None


def _set_callables(modules):
    """
    Set all Ansible modules callables
    :return:
    """

    def _set_function(cmd_name, doc):
        """
        Create a Salt function for the Ansible module.
        """

        def _cmd(*args, **kw):
            """
            Call an Ansible module as a function from the Salt.
            """
            kwargs = {}
            if kw.get("__pub_arg"):
                for _kw in kw.get("__pub_arg", []):
                    if isinstance(_kw, dict):
                        kwargs = _kw
                        break

            return _caller.call(cmd_name, *args, **kwargs)

        _cmd.__doc__ = doc
        return _cmd

    for mod in modules:
        setattr(sys.modules[__name__], mod, _set_function(mod, "Available"))


def __virtual__():
    """
    Ansible module caller.
    :return:
    """
    if salt.utils.platform.is_windows():
        return False, "The ansiblegate module isn't supported on Windows"
    ret = ansible is not None
    msg = not ret and "Ansible is not installed on this system" or None
    if ret:
        global _resolver
        global _caller
        _resolver = AnsibleModuleResolver(__opts__).resolve().install()
        _caller = AnsibleModuleCaller(_resolver)
        _set_callables(list())
    return __virtualname__


@depends("ansible")
def help(module=None, *args):
    """
    Display help on Ansible standard module.

    :param module:
    :return:
    """
    if not module:
        raise CommandExecutionError(
            "Please tell me what module you want to have helped with. "
            'Or call "ansible.list" to know what is available.'
        )
    try:
        module = _resolver.load_module(module)
    except (ImportError, LoaderError) as err:
        raise CommandExecutionError(
            'Module "{0}" is currently not functional on your system.'.format(module)
        )

    doc = {}
    ret = {}
    for docset in module.DOCUMENTATION.split("---"):
        try:
            docset = salt.utils.yaml.safe_load(docset)
            if docset:
                doc.update(docset)
        except Exception as err:  # pylint: disable=broad-except
            log.error("Error parsing doc section: %s", err)
    if not args:
        if "description" in doc:
            description = doc.get("description") or ""
            del doc["description"]
            ret["Description"] = description
        ret[
            'Available sections on module "{}"'.format(
                module.__name__.replace("ansible.modules.", "")
            )
        ] = list(doc)
    else:
        for arg in args:
            info = doc.get(arg)
            if info is not None:
                ret[arg] = info

    return ret


@depends("ansible")
def list_(pattern=None):
    """
    Lists available modules.
    :return:
    """
    return _resolver.get_modules_list(pattern=pattern)


@salt.utils.decorators.path.which("ansible-playbook")
def playbooks(
    playbook,
    rundir=None,
    check=False,
    diff=False,
    extra_vars=None,
    flush_cache=False,
    forks=5,
    inventory=None,
    limit=None,
    list_hosts=False,
    list_tags=False,
    list_tasks=False,
    module_path=None,
    skip_tags=None,
    start_at_task=None,
    syntax_check=False,
    tags=None,
    playbook_kwargs=None,
):
    """
    Run Ansible Playbooks

    :param playbook: Which playbook to run.
    :param rundir: Directory to run `ansible-playbook` in. (Default: None)
    :param check: don't make any changes; instead, try to predict some
                  of the changes that may occur (Default: False)
    :param diff: when changing (small) files and templates, show the
                 differences in those files; works great with --check
                 (default: False)
    :param extra_vars: set additional variables as key=value or YAML/JSON, if
                       filename prepend with @, (default: None)
    :param flush_cache: clear the fact cache for every host in inventory
                        (default: False)
    :param forks: specify number of parallel processes to use
                  (Default: 5)
    :param inventory: specify inventory host path or comma separated host
                      list. (Default: None) (Ansible's default is /etc/ansible/hosts)
    :param limit: further limit selected hosts to an additional pattern (Default: None)
    :param list_hosts: outputs a list of matching hosts; does not execute anything else
                       (Default: False)
    :param list_tags: list all available tags (Default: False)
    :param list_tasks: list all tasks that would be executed (Default: False)
    :param module_path: prepend colon-separated path(s) to module library. (Default: None)
    :param skip_tags: only run plays and tasks whose tags do not match these
                      values (Default: False)
    :param start_at_task: start the playbook at the task matching this name (Default: None)
    :param: syntax_check: perform a syntax check on the playbook, but do not execute it
                          (Default: False)
    :param tags: only run plays and tasks tagged with these values (Default: None)

    :return: Playbook return

    CLI Example:

    .. code-block:: bash

        salt 'ansiblehost'  ansible.playbook playbook=/srv/playbooks/play.yml
    """
    command = ["ansible-playbook", playbook]
    if check:
        command.append("--check")
    if diff:
        command.append("--diff")
    if isinstance(extra_vars, dict):
        command.append("--extra-vars='{0}'".format(json.dumps(extra_vars)))
    elif isinstance(extra_vars, six.text_type) and extra_vars.startswith("@"):
        command.append("--extra-vars={0}".format(extra_vars))
    if flush_cache:
        command.append("--flush-cache")
    if inventory:
        command.append("--inventory={0}".format(inventory))
    if limit:
        command.append("--limit={0}".format(limit))
    if list_hosts:
        command.append("--list-hosts")
    if list_tags:
        command.append("--list-tags")
    if list_tasks:
        command.append("--list-tasks")
    if module_path:
        command.append("--module-path={0}".format(module_path))
    if skip_tags:
        command.append("--skip-tags={0}".format(skip_tags))
    if start_at_task:
        command.append("--start-at-task={0}".format(start_at_task))
    if syntax_check:
        command.append("--syntax-check")
    if tags:
        command.append("--tags={0}".format(tags))
    if playbook_kwargs:
        for key, value in six.iteritems(playbook_kwargs):
            key = key.replace("_", "-")
            if value is True:
                command.append("--{0}".format(key))
            elif isinstance(value, six.text_type):
                command.append("--{0}={1}".format(key, value))
            elif isinstance(value, dict):
                command.append("--{0}={1}".format(key, json.dumps(value)))
    command.append("--forks={0}".format(forks))
    cmd_kwargs = {
        "env": {"ANSIBLE_STDOUT_CALLBACK": "json", "ANSIBLE_RETRY_FILES_ENABLED": "0"},
        "cwd": rundir,
        "cmd": " ".join(command),
    }
    ret = __salt__["cmd.run_all"](**cmd_kwargs)
    log.debug("Ansible Playbook Return: %s", ret)
    retdata = json.loads(ret["stdout"])
    if ret["retcode"]:
        __context__["retcode"] = ret["retcode"]
    return retdata



# -*- coding: utf-8 -*-
"""
This module allows you to manage assistive access on macOS minions with 10.9+

.. versionadded:: 2016.3.0

.. code-block:: bash

    salt '*' assistive.install /usr/bin/osascript
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import re

# Import salt libs
import salt.utils.platform
import salt.utils.stringutils
from salt.exceptions import CommandExecutionError
from salt.utils.versions import LooseVersion as _LooseVersion

log = logging.getLogger(__name__)

__virtualname__ = "assistive"


def __virtual__():
    """
    Only work on Mac OS
    """
    if not salt.utils.platform.is_darwin():
        return False, "Must be run on macOS"
    if _LooseVersion(__grains__["osrelease"]) < salt.utils.stringutils.to_str("10.9"):
        return False, "Must be run on macOS 10.9 or newer"
    return __virtualname__


def install(app_id, enable=True):
    """
    Install a bundle ID or command as being allowed to use
    assistive access.

    app_id
        The bundle ID or command to install for assistive access.

    enabled
        Sets enabled or disabled status. Default is ``True``.

    CLI Example:

    .. code-block:: bash

        salt '*' assistive.install /usr/bin/osascript
        salt '*' assistive.install com.smileonmymac.textexpander
    """
    ge_el_capitan = (
        True
        if _LooseVersion(__grains__["osrelease"])
        >= salt.utils.stringutils.to_str("10.11")
        else False
    )
    ge_mojave = (
        True
        if _LooseVersion(__grains__["osrelease"])
        >= salt.utils.stringutils.to_str("10.14")
        else False
    )
    client_type = _client_type(app_id)
    enable_str = "1" if enable else "0"
    cmd = (
        'sqlite3 "/Library/Application Support/com.apple.TCC/TCC.db" '
        "\"INSERT or REPLACE INTO access VALUES('kTCCServiceAccessibility','{0}',{1},{2},1,NULL{3}{4})\"".format(
            app_id,
            client_type,
            enable_str,
            ",NULL" if ge_el_capitan else "",
            ",NULL,NULL,NULL,NULL,''" if ge_mojave else "",
        )
    )

    call = __salt__["cmd.run_all"](cmd, output_loglevel="debug", python_shell=False)
    if call["retcode"] != 0:
        comment = ""
        if "stderr" in call:
            comment += call["stderr"]
        if "stdout" in call:
            comment += call["stdout"]

        raise CommandExecutionError("Error installing app: {0}".format(comment))

    return True


def installed(app_id):
    """
    Check if a bundle ID or command is listed in assistive access.
    This will not check to see if it's enabled.

    app_id
        The bundle ID or command to check installed status.

    CLI Example:

    .. code-block:: bash

        salt '*' assistive.installed /usr/bin/osascript
        salt '*' assistive.installed com.smileonmymac.textexpander
    """
    for a in _get_assistive_access():
        if app_id == a[0]:
            return True

    return False


def enable(app_id, enabled=True):
    """
    Enable or disable an existing assistive access application.

    app_id
        The bundle ID or command to set assistive access status.

    enabled
        Sets enabled or disabled status. Default is ``True``.

    CLI Example:

    .. code-block:: bash

        salt '*' assistive.enable /usr/bin/osascript
        salt '*' assistive.enable com.smileonmymac.textexpander enabled=False
    """
    enable_str = "1" if enabled else "0"
    for a in _get_assistive_access():
        if app_id == a[0]:
            cmd = (
                'sqlite3 "/Library/Application Support/com.apple.TCC/TCC.db" '
                "\"UPDATE access SET allowed='{0}' WHERE client='{1}'\"".format(
                    enable_str, app_id
                )
            )

            call = __salt__["cmd.run_all"](
                cmd, output_loglevel="debug", python_shell=False
            )

            if call["retcode"] != 0:
                comment = ""
                if "stderr" in call:
                    comment += call["stderr"]
                if "stdout" in call:
                    comment += call["stdout"]

                raise CommandExecutionError("Error enabling app: {0}".format(comment))

            return True

    return False


def enabled(app_id):
    """
    Check if a bundle ID or command is listed in assistive access and
    enabled.

    app_id
        The bundle ID or command to retrieve assistive access status.

    CLI Example:

    .. code-block:: bash

        salt '*' assistive.enabled /usr/bin/osascript
        salt '*' assistive.enabled com.smileonmymac.textexpander
    """
    for a in _get_assistive_access():
        if app_id == a[0] and a[1] == "1":
            return True

    return False


def remove(app_id):
    """
    Remove a bundle ID or command as being allowed to use assistive access.

    app_id
        The bundle ID or command to remove from assistive access list.

    CLI Example:

    .. code-block:: bash

        salt '*' assistive.remove /usr/bin/osascript
        salt '*' assistive.remove com.smileonmymac.textexpander
    """
    cmd = (
        'sqlite3 "/Library/Application Support/com.apple.TCC/TCC.db" '
        "\"DELETE from access where client='{0}'\"".format(app_id)
    )
    call = __salt__["cmd.run_all"](cmd, output_loglevel="debug", python_shell=False)

    if call["retcode"] != 0:
        comment = ""
        if "stderr" in call:
            comment += call["stderr"]
        if "stdout" in call:
            comment += call["stdout"]

        raise CommandExecutionError("Error removing app: {0}".format(comment))

    return True


def _client_type(app_id):
    """
    Determine whether the given ID is a bundle ID or a
    a path to a command
    """
    return "1" if app_id[0] == "/" else "0"


def _get_assistive_access():
    """
    Get a list of all of the assistive access applications installed,
    returns as a ternary showing whether each app is enabled or not.
    """
    cmd = 'sqlite3 "/Library/Application Support/com.apple.TCC/TCC.db" "SELECT * FROM access"'
    call = __salt__["cmd.run_all"](cmd, output_loglevel="debug", python_shell=False)

    if call["retcode"] != 0:
        comment = ""
        if "stderr" in call:
            comment += call["stderr"]
        if "stdout" in call:
            comment += call["stdout"]

        raise CommandExecutionError("Error: {0}".format(comment))

    out = call["stdout"]
    return re.findall(
        r"kTCCServiceAccessibility\|(.*)\|[0-9]{1}\|([0-9]{1})\|[0-9]{1}\|",
        out,
        re.MULTILINE,
    )



# -*- coding: utf-8 -*-
"""
This runner is used only for test purposes and servers no production purpose
"""
from __future__ import absolute_import, print_function, unicode_literals

# Import python libs
import time

from salt.ext import six
from salt.ext.six.moves import range


def arg(*args, **kwargs):
    """
    Output the given args and kwargs

    Kwargs will be filtered for 'private' keynames.
    """
    kwargs = dict((k, v) for k, v in six.iteritems(kwargs) if not k.startswith("__"))

    ret = {
        "args": args,
        "kwargs": kwargs,
    }
    return ret


def raw_arg(*args, **kwargs):
    """
    Output the given args and kwargs
    """
    ret = {
        "args": args,
        "kwargs": kwargs,
    }
    return ret


def metasyntactic(locality="us"):
    """
    Return common metasyntactic variables for the given locality
    """
    lookup = {
        "us": [
            "foo",
            "bar",
            "baz",
            "qux",
            "quux",
            "quuz",
            "corge",
            "grault",
            "garply",
            "waldo",
            "fred",
            "plugh",
            "xyzzy",
            "thud",
        ],
        "uk": ["wibble", "wobble", "wubble", "flob"],
    }
    return lookup.get(locality, None)


def stdout_print():
    """
    Print 'foo' and return 'bar'
    """
    print("foo")
    return "bar"


def sleep(s_time=10):
    """
    Sleep t seconds, then return True
    """
    print(s_time)
    time.sleep(s_time)
    return True


def stream():
    """
    Return True
    """
    ret = True
    for i in range(1, 100):
        __jid_event__.fire_event(
            {"message": "Runner is {0}% done".format(i)}, "progress"
        )
        time.sleep(0.1)
    return ret


def get_opts():
    """
    .. versionadded:: 2018.3.0

    Return the configuration options of the master.

    CLI Example:

    .. code-block:: bash

        salt-run test.get_opts
    """
    return __opts__



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.svn as svn

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SvnTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.svn
    """

    def setup_loader_modules(self):
        return {svn: {}}

    def test_info(self):
        """
        Test to display the Subversion information from the checkout.
        """
        mock = MagicMock(
            side_effect=[
                {"retcode": 0, "stdout": True},
                {"retcode": 0, "stdout": "A\n\nB"},
                {"retcode": 0, "stdout": "A\n\nB"},
            ]
        )
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.info("cwd", fmt="xml"))

            self.assertListEqual(svn.info("cwd", fmt="list"), [[], []])

            self.assertListEqual(svn.info("cwd", fmt="dict"), [{}, {}])

    def test_checkout(self):
        """
        Test to download a working copy of the remote Subversion repository
        directory or file
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.checkout("cwd", "remote"))

    def test_switch(self):
        """
        Test to switch a working copy of a remote Subversion repository
        directory
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.switch("cwd", "remote"))

    def test_update(self):
        """
        Test to update the current directory, files, or directories from
        the remote Subversion repository
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.update("cwd"))

    def test_diff(self):
        """
        Test to return the diff of the current directory, files, or
        directories from the remote Subversion repository
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.diff("cwd"))

    def test_commit(self):
        """
        Test to commit the current directory, files, or directories to
        the remote Subversion repository
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.commit("cwd"))

    def test_add(self):
        """
        Test to add files to be tracked by the Subversion working-copy
        checkout
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.add("cwd", False))

    def test_remove(self):
        """
        Test to remove files and directories from the Subversion repository
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.remove("cwd", False))

    def test_status(self):
        """
        Test to display the status of the current directory, files, or
        directories in the Subversion repository
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.status("cwd"))

    def test_export(self):
        """
        Test to create an unversioned copy of a tree.
        """
        mock = MagicMock(return_value={"retcode": 0, "stdout": True})
        with patch.dict(svn.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(svn.export("cwd", "remote"))



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.proxy as proxy

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, call, patch
from tests.support.unit import TestCase


class ProxyTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.proxy
    """

    def setup_loader_modules(self):
        return {proxy: {"__grains__": {"os": "Darwin"}}}

    def test_get_http_proxy_macos(self):
        """
            Test to make sure that we correctly get the current proxy info
            on macOS
        """
        mock = MagicMock(
            return_value="Enabled: Yes\nServer: 192.168.0.1\nPort: 3128\nAuthenticated Proxy Enabled: 0"
        )
        expected = {"enabled": True, "server": "192.168.0.1", "port": "3128"}

        with patch.dict(proxy.__salt__, {"cmd.run": mock}):
            out = proxy.get_http_proxy()
            mock.assert_called_once_with("networksetup -getwebproxy Ethernet")
            self.assertEqual(expected, out)

    def test_get_https_proxy_macos(self):
        """
            Test to make sure that we correctly get the current proxy info
            on macOS
        """
        mock = MagicMock(
            return_value="Enabled: Yes\nServer: 192.168.0.1\nPort: 3128\nAuthenticated Proxy Enabled: 0"
        )
        expected = {"enabled": True, "server": "192.168.0.1", "port": "3128"}

        with patch.dict(proxy.__salt__, {"cmd.run": mock}):
            out = proxy.get_https_proxy()
            mock.assert_called_once_with("networksetup -getsecurewebproxy Ethernet")
            self.assertEqual(expected, out)

    def test_get_ftp_proxy_macos(self):
        """
            Test to make sure that we correctly get the current proxy info
            on macOS
        """
        mock = MagicMock(
            return_value="Enabled: Yes\nServer: 192.168.0.1\nPort: 3128\nAuthenticated Proxy Enabled: 0"
        )
        expected = {"enabled": True, "server": "192.168.0.1", "port": "3128"}

        with patch.dict(proxy.__salt__, {"cmd.run": mock}):
            out = proxy.get_ftp_proxy()
            mock.assert_called_once_with("networksetup -getftpproxy Ethernet")
            self.assertEqual(expected, out)

    def test_get_http_proxy_macos_none(self):
        """
            Test to make sure that we correctly return when there's no proxy set
        """
        mock = MagicMock(
            return_value="Enabled: No\nServer:\nPort: 0\nAuthenticated Proxy Enabled: 0"
        )

        with patch.dict(proxy.__salt__, {"cmd.run": mock}):
            out = proxy.get_http_proxy()
            mock.assert_called_once_with("networksetup -getwebproxy Ethernet")
            self.assertEqual({}, out)

    def test_set_http_proxy_macos(self):
        """
            Test to make sure that we correctly set the proxy info
            on macOS
        """
        mock = MagicMock()

        with patch.dict(proxy.__salt__, {"cmd.run": mock}):
            out = proxy.set_http_proxy(
                "192.168.0.1",
                3128,
                "frank",
                "badpassw0rd",
                bypass_hosts=".moo.com,.salt.com",
            )
            mock.assert_called_once_with(
                "networksetup -setwebproxy Ethernet 192.168.0.1 3128 On frank badpassw0rd"
            )
            self.assertTrue(out)

    def test_set_https_proxy_macos(self):
        """
            Test to make sure that we correctly set the proxy info
            on macOS
        """
        mock = MagicMock()

        with patch.dict(proxy.__salt__, {"cmd.run": mock}):
            out = proxy.set_https_proxy(
                "192.168.0.1",
                3128,
                "frank",
                "passw0rd",
                bypass_hosts=".moo.com,.salt.com",
            )
            mock.assert_called_once_with(
                "networksetup -setsecurewebproxy Ethernet 192.168.0.1 3128 On frank passw0rd"
            )
            self.assertTrue(out)

    def test_set_ftp_proxy_macos(self):
        """
            Test to make sure that we correctly set the proxy info
            on macOS
        """
        mock = MagicMock()

        with patch.dict(proxy.__salt__, {"cmd.run": mock}):
            out = proxy.set_ftp_proxy(
                "192.168.0.1",
                3128,
                "frank",
                "badpassw0rd",
                bypass_hosts=".moo.com,.salt.com",
            )
            mock.assert_called_once_with(
                "networksetup -setftpproxy Ethernet 192.168.0.1 3128 On frank badpassw0rd"
            )
            self.assertTrue(out)

    def test_get_proxy_win(self):
        """
        Test to make sure that we correctly get the current proxy info on
        Windows
        """
        result = [
            {
                "vdata": "http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128"
            },
            {"vdata": 1},
        ]
        mock_reg_read = MagicMock(side_effect=result)
        expected = {
            "enabled": True,
            "http": {"server": "192.168.0.1", "port": "3128"},
            "https": {"server": "192.168.0.2", "port": "3128"},
            "ftp": {"server": "192.168.0.3", "port": "3128"},
        }
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.read_value": mock_reg_read}
        ):
            out = proxy.get_proxy_win()
            self.assertDictEqual(out, expected)
            mock_reg_read.assert_any_call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
            )
            mock_reg_read.assert_any_call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyEnable",
            )

    def test_get_http_proxy_windows(self):
        """
        Test to make sure that we correctly get the current proxy info on
        Windows
        """
        result = {
            "vdata": "http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128"
        }
        mock = MagicMock(return_value=result)
        expected = {"server": "192.168.0.1", "port": "3128"}
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.read_value": mock}
        ):
            out = proxy.get_http_proxy()
            mock.assert_called_once_with(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
            )
            self.assertEqual(expected, out)

    def test_get_https_proxy_windows(self):
        """
        Test to make sure that we correctly get the current proxy info on
        Windows
        """
        result = {
            "vdata": "http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128"
        }
        mock = MagicMock(return_value=result)
        expected = {"server": "192.168.0.2", "port": "3128"}
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.read_value": mock}
        ):
            out = proxy.get_https_proxy()
            mock.assert_called_once_with(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
            )
            self.assertEqual(expected, out)

    def test_get_ftp_proxy_windows(self):
        """
        Test to make sure that we correctly get the current proxy info on
        Windows
        """
        result = {
            "vdata": "http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128"
        }
        mock = MagicMock(return_value=result)
        expected = {"server": "192.168.0.3", "port": "3128"}
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.read_value": mock}
        ):
            out = proxy.get_ftp_proxy()
            mock.assert_called_once_with(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
            )
            self.assertEqual(expected, out)

    def test_get_all_proxies_macos_fails(self):
        mock = MagicMock()
        with patch.dict(proxy.__utils__, {"reg.read_value": mock}):
            out = proxy.get_proxy_win()
            assert not mock.called
            self.assertEqual(out, None)

    def test_get_all_proxies_windows(self):
        """
        Test to make sure that we correctly get the current proxy info on
        Windows
        """
        results = [
            {
                "vdata": "http=192.168.0.1:3128;https=192.168.0.2:3128;ftp=192.168.0.3:3128"
            },
            {"vdata": 1},
        ]
        mock = MagicMock(side_effect=results)
        expected = {
            "enabled": True,
            "http": {"server": "192.168.0.1", "port": "3128"},
            "https": {"server": "192.168.0.2", "port": "3128"},
            "ftp": {"server": "192.168.0.3", "port": "3128"},
        }
        calls = [
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyEnable",
            ),
        ]
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.read_value": mock}
        ):
            out = proxy.get_proxy_win()
            mock.assert_has_calls(calls)
            self.assertEqual(expected, out)

    def test_set_http_proxy_windows(self):
        """
        Test to make sure that we correctly set the proxy info on Windows
        """
        calls = [
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
                vdata="http=192.168.0.1:3128;",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyEnable",
                vdata=1,
                vtype="REG_DWORD",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyOverride",
                vdata="<local>;.moo.com;.salt.com",
            ),
        ]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.set_value": mock_reg}
        ), patch.dict(proxy.__salt__, {"cmd.run": mock_cmd}):
            out = proxy.set_http_proxy(
                server="192.168.0.1", port=3128, bypass_hosts=[".moo.com", ".salt.com"],
            )
            mock_reg.assert_has_calls(calls)
            mock_cmd.assert_called_once_with("netsh winhttp import proxy source=ie")
            self.assertTrue(out)

    def test_set_https_proxy_windows(self):
        """
        Test to make sure that we correctly set the proxy info on Windows
        """
        calls = [
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
                vdata="https=192.168.0.1:3128;",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyEnable",
                vdata=1,
                vtype="REG_DWORD",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyOverride",
                vdata="<local>;.moo.com;.salt.com",
            ),
        ]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.set_value": mock_reg}
        ), patch.dict(proxy.__salt__, {"cmd.run": mock_cmd}):
            out = proxy.set_https_proxy(
                server="192.168.0.1", port=3128, bypass_hosts=[".moo.com", ".salt.com"],
            )
            mock_reg.assert_has_calls(calls)
            mock_cmd.assert_called_once_with("netsh winhttp import proxy source=ie")
            self.assertTrue(out)

    def test_set_ftp_proxy_windows(self):
        """
        Test to make sure that we correctly set the proxy info on Windows
        """
        calls = [
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
                vdata="ftp=192.168.0.1:3128;",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyEnable",
                vdata=1,
                vtype="REG_DWORD",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyOverride",
                vdata="<local>;.moo.com;.salt.com",
            ),
        ]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.set_value": mock_reg}
        ), patch.dict(proxy.__salt__, {"cmd.run": mock_cmd}):
            out = proxy.set_ftp_proxy(
                server="192.168.0.1", port=3128, bypass_hosts=[".moo.com", ".salt.com"],
            )
            mock_reg.assert_has_calls(calls)
            mock_cmd.assert_called_once_with("netsh winhttp import proxy source=ie")
            self.assertTrue(out)

    def test_set_proxy_windows(self):
        """
        Test to make sure that we correctly set the proxy info on Windows
        """
        calls = [
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
                vdata="http=192.168.0.1:3128;https=192.168.0.1:3128;ftp=192.168.0.1:3128;",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyEnable",
                vdata=1,
                vtype="REG_DWORD",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyOverride",
                vdata="<local>;.moo.com;.salt.com",
            ),
        ]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.set_value": mock_reg}
        ), patch.dict(proxy.__salt__, {"cmd.run": mock_cmd}):
            out = proxy.set_proxy_win(
                server="192.168.0.1", port=3128, bypass_hosts=[".moo.com", ".salt.com"],
            )
            mock_reg.assert_has_calls(calls)
            mock_cmd.assert_called_once_with("netsh winhttp import proxy source=ie")
            self.assertTrue(out)

    def test_set_proxy_windows_no_ftp(self):
        """
        Test to make sure that we correctly set the proxy info on Windows
        """
        calls = [
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyServer",
                vdata="http=192.168.0.1:3128;https=192.168.0.1:3128;",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyEnable",
                vdata=1,
                vtype="REG_DWORD",
            ),
            call(
                hive="HKEY_CURRENT_USER",
                key="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
                vname="ProxyOverride",
                vdata="<local>;.moo.com;.salt.com",
            ),
        ]
        mock_reg = MagicMock()
        mock_cmd = MagicMock()
        with patch.dict(proxy.__grains__, {"os": "Windows"}), patch.dict(
            proxy.__utils__, {"reg.set_value": mock_reg}
        ), patch.dict(proxy.__salt__, {"cmd.run": mock_cmd}):
            out = proxy.set_proxy_win(
                server="192.168.0.1",
                port=3128,
                types=["http", "https"],
                bypass_hosts=[".moo.com", ".salt.com"],
            )
            mock_reg.assert_has_calls(calls)
            mock_cmd.assert_called_once_with("netsh winhttp import proxy source=ie")
            self.assertTrue(out)



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

from textwrap import dedent

# Import Salt libs
import salt.renderers.gpg as gpg
from salt.exceptions import SaltRenderError

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class GPGTestCase(TestCase, LoaderModuleMockMixin):
    """
    unit test GPG renderer
    """

    def setup_loader_modules(self):
        return {gpg: {}}

    def test__get_gpg_exec(self):
        """
        test _get_gpg_exec
        """
        gpg_exec = "/bin/gpg"

        with patch("salt.utils.path.which", MagicMock(return_value=gpg_exec)):
            self.assertEqual(gpg._get_gpg_exec(), gpg_exec)

        with patch("salt.utils.path.which", MagicMock(return_value=False)):
            self.assertRaises(SaltRenderError, gpg._get_gpg_exec)

    def test__decrypt_ciphertext(self):
        """
        test _decrypt_ciphertext
        """
        key_dir = "/etc/salt/gpgkeys"
        secret = "Use more salt."
        crypted = "-----BEGIN PGP MESSAGE-----!@#$%^&*()_+-----END PGP MESSAGE-----"

        multisecret = "password is {0} and salt is {0}".format(secret)
        multicrypted = "password is {0} and salt is {0}".format(crypted)

        class GPGDecrypt(object):
            def communicate(self, *args, **kwargs):
                return [secret, None]

        class GPGNotDecrypt(object):
            def communicate(self, *args, **kwargs):
                return [None, "decrypt error"]

        with patch(
            "salt.renderers.gpg._get_key_dir", MagicMock(return_value=key_dir)
        ), patch("salt.utils.path.which", MagicMock()):
            with patch(
                "salt.renderers.gpg.Popen", MagicMock(return_value=GPGDecrypt())
            ):
                self.assertEqual(gpg._decrypt_ciphertexts(crypted), secret)
                self.assertEqual(gpg._decrypt_ciphertexts(multicrypted), multisecret)
            with patch(
                "salt.renderers.gpg.Popen", MagicMock(return_value=GPGNotDecrypt())
            ):
                self.assertEqual(gpg._decrypt_ciphertexts(crypted), crypted)
                self.assertEqual(gpg._decrypt_ciphertexts(multicrypted), multicrypted)

    def test__decrypt_object(self):
        """
        test _decrypt_object
        """
        secret = "Use more salt."
        crypted = "-----BEGIN PGP MESSAGE-----!@#$%^&*()_+-----END PGP MESSAGE-----"

        secret_map = {"secret": secret}
        crypted_map = {"secret": crypted}

        secret_list = [secret]
        crypted_list = [crypted]

        with patch(
            "salt.renderers.gpg._decrypt_ciphertext", MagicMock(return_value=secret)
        ):
            self.assertEqual(gpg._decrypt_object(secret), secret)
            self.assertEqual(gpg._decrypt_object(crypted), secret)
            self.assertEqual(gpg._decrypt_object(crypted_map), secret_map)
            self.assertEqual(gpg._decrypt_object(crypted_list), secret_list)
            self.assertEqual(gpg._decrypt_object(None), None)

    def test_render(self):
        """
        test render
        """
        key_dir = "/etc/salt/gpgkeys"
        secret = "Use more salt."
        crypted = "-----BEGIN PGP MESSAGE-----!@#$%^&*()_+"

        with patch("salt.renderers.gpg._get_gpg_exec", MagicMock(return_value=True)):
            with patch(
                "salt.renderers.gpg._get_key_dir", MagicMock(return_value=key_dir)
            ):
                with patch(
                    "salt.renderers.gpg._decrypt_object", MagicMock(return_value=secret)
                ):
                    self.assertEqual(gpg.render(crypted), secret)

    def test_multi_render(self):
        key_dir = "/etc/salt/gpgkeys"
        secret = "Use more salt."
        expected = "\n".join([secret] * 3)
        crypted = dedent(
            """\
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----
        """
        )

        with patch("salt.renderers.gpg._get_gpg_exec", MagicMock(return_value=True)):
            with patch(
                "salt.renderers.gpg._get_key_dir", MagicMock(return_value=key_dir)
            ):
                with patch(
                    "salt.renderers.gpg._decrypt_ciphertext",
                    MagicMock(return_value=secret),
                ):
                    self.assertEqual(gpg.render(crypted), expected)

    def test_render_with_binary_data_should_return_binary_data(self):
        key_dir = "/etc/salt/gpgkeys"
        secret = b"Use\x8b more\x8b salt."
        expected = b"\n".join([secret] * 3)
        crypted = dedent(
            """\
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----
        """
        )

        with patch("salt.renderers.gpg._get_gpg_exec", MagicMock(return_value=True)):
            with patch(
                "salt.renderers.gpg._get_key_dir", MagicMock(return_value=key_dir)
            ):
                with patch(
                    "salt.renderers.gpg._decrypt_ciphertext",
                    MagicMock(return_value=secret),
                ):
                    self.assertEqual(gpg.render(crypted, encoding="utf-8"), expected)

    def test_render_with_translate_newlines_should_translate_newlines(self):
        key_dir = "/etc/salt/gpgkeys"
        secret = b"Use\x8b more\x8b salt."
        expected = b"\n\n".join([secret] * 3)
        crypted = dedent(
            """\
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----\\n
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----\\n
            -----BEGIN PGP MESSAGE-----
            !@#$%^&*()_+
            -----END PGP MESSAGE-----
        """
        )

        with patch("salt.renderers.gpg._get_gpg_exec", MagicMock(return_value=True)):
            with patch(
                "salt.renderers.gpg._get_key_dir", MagicMock(return_value=key_dir)
            ):
                with patch(
                    "salt.renderers.gpg._decrypt_ciphertext",
                    MagicMock(return_value=secret),
                ):
                    self.assertEqual(
                        gpg.render(crypted, translate_newlines=True, encoding="utf-8"),
                        expected,
                    )



# coding: utf-8

# Python libs
from __future__ import absolute_import

from collections import namedtuple

# Salt libs
import salt.beacons.diskusage as diskusage
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock, patch

# Salt testing libs
from tests.support.unit import TestCase

STUB_DISK_PARTITION = [
    namedtuple("partition", "device mountpoint fstype, opts")(
        "tmpfs", "/mnt/tmp", "tmpfs", "rw,nosuid,nodev,relatime,size=10240k"
    ),
    namedtuple("partition", "device mountpoint fstype, opts")(
        "/dev/disk0s2", "/", "hfs", "rw,local,rootfs,dovolfs,journaled,multilabel"
    ),
]

WINDOWS_STUB_DISK_PARTITION = [
    namedtuple("partition", "device mountpoint fstype, opts")(
        "C:\\", "C:\\", "NTFS", "rw,fixed"
    ),
    namedtuple("partition", "device mountpoint fstype, opts")(
        "D:\\", "D:\\", "CDFS", "ro,cdrom"
    ),
]

STUB_DISK_USAGE = [
    namedtuple("usage", "total used free percent")(1000, 500, 500, 50),
    namedtuple("usage", "total used free percent")(100, 75, 25, 25),
]

WINDOWS_STUB_DISK_USAGE = namedtuple("usage", "total used free percent")(
    1000, 500, 500, 50
)


class DiskUsageBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.adb
    """

    def setup_loader_modules(self):
        return {}

    def test_non_list_config(self):
        config = {}

        ret = diskusage.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for diskusage beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = diskusage.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

    def test_diskusage_match(self):
        disk_usage_mock = Mock(side_effect=STUB_DISK_USAGE)
        with patch(
            "salt.utils.platform.is_windows", MagicMock(return_value=False)
        ), patch(
            "psutil.disk_partitions", MagicMock(return_value=STUB_DISK_PARTITION)
        ), patch(
            "psutil.disk_usage", disk_usage_mock
        ):
            config = [{"/": "50%"}]

            ret = diskusage.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = diskusage.beacon(config)
            self.assertEqual(ret, [{"diskusage": 50, "mount": "/"}])

    def test_diskusage_nomatch(self):
        disk_usage_mock = Mock(side_effect=STUB_DISK_USAGE)
        with patch(
            "salt.utils.platform.is_windows", MagicMock(return_value=False)
        ), patch(
            "psutil.disk_partitions", MagicMock(return_value=STUB_DISK_PARTITION)
        ), patch(
            "psutil.disk_usage", disk_usage_mock
        ):
            config = [{"/": "70%"}]

            ret = diskusage.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = diskusage.beacon(config)
            self.assertNotEqual(ret, [{"diskusage": 50, "mount": "/"}])

    def test_diskusage_match_regex(self):
        disk_usage_mock = Mock(side_effect=STUB_DISK_USAGE)
        with patch(
            "salt.utils.platform.is_windows", MagicMock(return_value=False)
        ), patch(
            "psutil.disk_partitions", MagicMock(return_value=STUB_DISK_PARTITION)
        ), patch(
            "psutil.disk_usage", disk_usage_mock
        ):
            config = [{r"^\/": "50%"}]

            ret = diskusage.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = diskusage.beacon(config)
            self.assertEqual(ret, [{"diskusage": 50, "mount": "/"}])

    def test_diskusage_windows_single_slash(self):
        r"""
        This tests new behavior (C:\)
        """
        disk_usage_mock = Mock(return_value=WINDOWS_STUB_DISK_USAGE)
        with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
            with patch(
                "psutil.disk_partitions",
                MagicMock(return_value=WINDOWS_STUB_DISK_PARTITION),
            ), patch("psutil.disk_usage", disk_usage_mock):
                config = [{"C:\\": "50%"}]

                ret = diskusage.validate(config)

                self.assertEqual(ret, (True, "Valid beacon configuration"))

                ret = diskusage.beacon(config)
                self.assertEqual(ret, [{"diskusage": 50, "mount": "C:\\"}])

    def test_diskusage_windows_double_slash(self):
        """
        This tests original behavior (C:\\)
        """
        disk_usage_mock = Mock(return_value=WINDOWS_STUB_DISK_USAGE)
        with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
            with patch(
                "psutil.disk_partitions",
                MagicMock(return_value=WINDOWS_STUB_DISK_PARTITION),
            ), patch("psutil.disk_usage", disk_usage_mock):
                config = [{"C:\\\\": "50%"}]

                ret = diskusage.validate(config)

                self.assertEqual(ret, (True, "Valid beacon configuration"))

                ret = diskusage.beacon(config)
                self.assertEqual(ret, [{"diskusage": 50, "mount": "C:\\"}])

    def test_diskusage_windows_lowercase(self):
        r"""
        This tests lowercase drive letter (c:\)
        """
        disk_usage_mock = Mock(return_value=WINDOWS_STUB_DISK_USAGE)
        with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
            with patch(
                "psutil.disk_partitions",
                MagicMock(return_value=WINDOWS_STUB_DISK_PARTITION),
            ), patch("psutil.disk_usage", disk_usage_mock):
                config = [{"c:\\": "50%"}]

                ret = diskusage.validate(config)

                self.assertEqual(ret, (True, "Valid beacon configuration"))

                ret = diskusage.beacon(config)
                self.assertEqual(ret, [{"diskusage": 50, "mount": "C:\\"}])

    def test_diskusage_windows_match_regex(self):
        disk_usage_mock = Mock(return_value=WINDOWS_STUB_DISK_USAGE)
        with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
            with patch(
                "psutil.disk_partitions",
                MagicMock(return_value=WINDOWS_STUB_DISK_PARTITION),
            ), patch("psutil.disk_usage", disk_usage_mock):
                config = [{"^[a-zA-Z]:\\": "50%"}]

                ret = diskusage.validate(config)

                self.assertEqual(ret, (True, "Valid beacon configuration"))

                ret = diskusage.beacon(config)
                _expected = [
                    {u"diskusage": 50, u"mount": "C:\\"},
                    {u"diskusage": 50, u"mount": "D:\\"},
                ]
                self.assertEqual(ret, _expected)



# -*- coding: utf-8 -*-
"""
Integration tests for the docker_container states
"""
from __future__ import absolute_import, print_function, unicode_literals

import random
import string
import sys

import salt.utils.path
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest, slowTest
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Testing Libs
from tests.support.unit import skipIf


def _random_name(prefix=""):
    ret = prefix
    for _ in range(8):
        ret += random.choice(string.ascii_lowercase)
    return ret


@destructiveTest
@skipIf(not salt.utils.path.which("dockerd"), "Docker not installed")
class DockerCallTestCase(ModuleCase, SaltReturnAssertsMixin):
    """
    Test docker_container states
    """

    def setUp(self):
        """
        setup docker.call tests
        """
        # Create temp dir
        self.random_name = _random_name(prefix="salt_test_")
        self.image_tag = sys.version_info[0]

        self.run_state("docker_image.present", tag=self.image_tag, name="python")
        self.run_state(
            "docker_container.running",
            name=self.random_name,
            image="python:{0}".format(self.image_tag),
            entrypoint="tail -f /dev/null",
        )

    def tearDown(self):
        """
        teardown docker.call tests
        """
        self.run_state("docker_container.absent", name=self.random_name, force=True)
        self.run_state(
            "docker_image.absent",
            images=["python:{0}".format(self.image_tag)],
            force=True,
        )
        delattr(self, "random_name")
        delattr(self, "image_tag")

    @slowTest
    def test_docker_call(self):
        """
        check that docker.call works, and works with a container not running as root
        """
        ret = self.run_function("docker.call", [self.random_name, "test.ping"])
        assert ret is True

    @slowTest
    def test_docker_sls(self):
        """
        check that docker.sls works, and works with a container not running as root
        """
        ret = self.run_function("docker.apply", [self.random_name, "core"])
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_docker_highstate(self):
        """
        check that docker.highstate works, and works with a container not running as root
        """
        ret = self.run_function("docker.apply", [self.random_name])
        self.assertSaltTrueReturn(ret)



# -*- coding: utf-8 -*-
"""
Integration tests for the alternatives state module
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest, slowTest
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

NO_ALTERNATIVES = False
if not os.path.exists("/etc/alternatives"):
    NO_ALTERNATIVES = True


@skipIf(NO_ALTERNATIVES, "/etc/alternatives does not exist on the system")
class AlterantivesStateTest(ModuleCase, SaltReturnAssertsMixin):
    @destructiveTest
    @slowTest
    def test_install_set_and_remove(self):
        ret = self.run_state(
            "alternatives.set", name="alt-test", path=RUNTIME_VARS.SHELL_TRUE_PATH
        )
        self.assertSaltFalseReturn(ret)

        ret = self.run_state(
            "alternatives.install",
            name="alt-test",
            link="/usr/local/bin/alt-test",
            path=RUNTIME_VARS.SHELL_TRUE_PATH,
            priority=50,
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(
            ret, RUNTIME_VARS.SHELL_TRUE_PATH, keys=["path"]
        )

        ret = self.run_state(
            "alternatives.install",
            name="alt-test",
            link="/usr/local/bin/alt-test",
            path=RUNTIME_VARS.SHELL_TRUE_PATH,
            priority=50,
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(ret, {})

        ret = self.run_state(
            "alternatives.install",
            name="alt-test",
            link="/usr/local/bin/alt-test",
            path=RUNTIME_VARS.SHELL_FALSE_PATH,
            priority=90,
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(
            ret, RUNTIME_VARS.SHELL_FALSE_PATH, keys=["path"]
        )

        ret = self.run_state(
            "alternatives.set", name="alt-test", path=RUNTIME_VARS.SHELL_FALSE_PATH
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(ret, {})

        ret = self.run_state(
            "alternatives.set", name="alt-test", path=RUNTIME_VARS.SHELL_TRUE_PATH
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(
            ret, RUNTIME_VARS.SHELL_TRUE_PATH, keys=["path"]
        )

        ret = self.run_state(
            "alternatives.set", name="alt-test", path=RUNTIME_VARS.SHELL_TRUE_PATH
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(ret, {})

        ret = self.run_state(
            "alternatives.remove", name="alt-test", path=RUNTIME_VARS.SHELL_TRUE_PATH
        )
        self.assertSaltTrueReturn(ret)

        ret = self.run_state(
            "alternatives.remove", name="alt-test", path=RUNTIME_VARS.SHELL_FALSE_PATH
        )
        self.assertSaltTrueReturn(ret)
